@@ -19,18 +19,18 @@ import (
 	"crypto/tls"
 	"encoding/hex"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"regexp"
 	"strconv"
 	"strings"
 	"time"
 
-	pb "go.etcd.io/etcd/api/v3/mvccpb"
-	v3 "go.etcd.io/etcd/client/v3"
-	"go.etcd.io/etcd/pkg/v3/cobrautl"
-
 	"github.com/spf13/cobra"
+
+	pb "go.etcd.io/etcd/api/v3/mvccpb"
+	clientv3 "go.etcd.io/etcd/client/v3"
+	"go.etcd.io/etcd/pkg/v3/cobrautl"
 )
 
 func printKV(isHex bool, valueOnly bool, kv *pb.KeyValue) {
@@ -56,9 +56,10 @@ func addHexPrefix(s string) string {
 	return string(ns)
 }
 
-func argify(s string) []string {
-	r := regexp.MustCompile(`"(?:[^"\\]|\\.)*"|'[^']*'|[^'"\s]\S*[^'"\s]?`)
-	args := r.FindAllString(s, -1)
+var argsRegexp = regexp.MustCompile(`"(?:[^"\\]|\\.)*"|'[^']*'|[^'"\s]\S*[^'"\s]?`)
+
+func Argify(s string) []string {
+	args := argsRegexp.FindAllString(s, -1)
 	for i := range args {
 		if len(args[i]) == 0 {
 			continue
@@ -93,7 +94,7 @@ func isCommandTimeoutFlagSet(cmd *cobra.Command) bool {
 }
 
 // get the process_resident_memory_bytes from <server>/metrics
-func endpointMemoryMetrics(host string, scfg *secureCfg) float64 {
+func endpointMemoryMetrics(host string, scfg *clientv3.SecureConfig) float64 {
 	residentMemoryKey := "process_resident_memory_bytes"
 	var residentMemoryValue string
 	if !strings.HasPrefix(host, "http://") && !strings.HasPrefix(host, "https://") {
@@ -102,25 +103,25 @@ func endpointMemoryMetrics(host string, scfg *secureCfg) float64 {
 	url := host + "/metrics"
 	if strings.HasPrefix(host, "https://") {
 		// load client certificate
-		cert, err := tls.LoadX509KeyPair(scfg.cert, scfg.key)
+		cert, err := tls.LoadX509KeyPair(scfg.Cert, scfg.Key)
 		if err != nil {
-			fmt.Println(fmt.Sprintf("client certificate error: %v", err))
+			fmt.Printf("client certificate error: %v\n", err)
 			return 0.0
 		}
 		http.DefaultTransport.(*http.Transport).TLSClientConfig = &tls.Config{
 			Certificates:       []tls.Certificate{cert},
-			InsecureSkipVerify: scfg.insecureSkipVerify,
+			InsecureSkipVerify: scfg.InsecureSkipVerify,
 		}
 	}
 	resp, err := http.Get(url)
 	if err != nil {
-		fmt.Println(fmt.Sprintf("fetch error: %v", err))
+		fmt.Printf("fetch error: %v\n", err)
 		return 0.0
 	}
-	byts, readerr := ioutil.ReadAll(resp.Body)
+	byts, readerr := io.ReadAll(resp.Body)
 	resp.Body.Close()
 	if readerr != nil {
-		fmt.Println(fmt.Sprintf("fetch error: reading %s: %v", url, readerr))
+		fmt.Printf("fetch error: reading %s: %v\n", url, readerr)
 		return 0.0
 	}
 
@@ -131,12 +132,12 @@ func endpointMemoryMetrics(host string, scfg *secureCfg) float64 {
 		}
 	}
 	if residentMemoryValue == "" {
-		fmt.Println(fmt.Sprintf("could not find: %v", residentMemoryKey))
+		fmt.Printf("could not find: %v\n", residentMemoryKey)
 		return 0.0
 	}
 	residentMemoryBytes, parseErr := strconv.ParseFloat(residentMemoryValue, 64)
 	if parseErr != nil {
-		fmt.Println(fmt.Sprintf("parse error: %v", parseErr))
+		fmt.Printf("parse error: %v\n", parseErr)
 		return 0.0
 	}
 
@@ -144,10 +145,10 @@ func endpointMemoryMetrics(host string, scfg *secureCfg) float64 {
 }
 
 // compact keyspace history to a provided revision
-func compact(c *v3.Client, rev int64) {
+func compact(c *clientv3.Client, rev int64) {
 	fmt.Printf("Compacting with revision %d\n", rev)
 	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
-	_, err := c.Compact(ctx, rev, v3.WithCompactPhysical())
+	_, err := c.Compact(ctx, rev, clientv3.WithCompactPhysical())
 	cancel()
 	if err != nil {
 		cobrautl.ExitWithError(cobrautl.ExitError, err)
@@ -156,7 +157,7 @@ func compact(c *v3.Client, rev int64) {
 }
 
 // defrag a given endpoint
-func defrag(c *v3.Client, ep string) {
+func defrag(c *clientv3.Client, ep string) {
 	fmt.Printf("Defragmenting %q\n", ep)
 	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
 	_, err := c.Defragment(ctx, ep)
@@ -166,3 +167,14 @@ func defrag(c *v3.Client, ep string) {
 	}
 	fmt.Printf("Defragmented %q\n", ep)
 }
+
+func IsSerializable(option string) bool {
+	switch option {
+	case "s":
+		return true
+	case "l":
+	default:
+		cobrautl.ExitWithError(cobrautl.ExitBadFeature, fmt.Errorf("unknown consistency flag %q", getConsistency))
+	}
+	return false
+}
