@@ -27,4 +27,11 @@ func (p *fieldsPrinter) DBStatus(r snapshot.Status) {
 	fmt.Println(`"Revision" :`, r.Revision)
 	fmt.Println(`"Keys" :`, r.TotalKey)
 	fmt.Println(`"Size" :`, r.TotalSize)
+	fmt.Println(`"Version" :`, r.Version)
+}
+
+func (p *fieldsPrinter) DBHashKV(r HashKV) {
+	fmt.Println(`"Hash" :`, r.Hash)
+	fmt.Println(`"Hash revision" :`, r.HashRevision)
+	fmt.Println(`"Compact revision" :`, r.CompactRevision)
 }
