@@ -33,9 +33,10 @@ func newJSONPrinter() printer {
 }
 
 func (p *jsonPrinter) DBStatus(r snapshot.Status) { printJSON(r) }
+func (p *jsonPrinter) DBHashKV(r HashKV)          { printJSON(r) }
 
 // !!! Share ??
-func printJSON(v interface{}) {
+func printJSON(v any) {
 	b, err := json.Marshal(v)
 	if err != nil {
 		fmt.Fprintf(os.Stderr, "%v\n", err)
