@@ -19,7 +19,6 @@ import (
 	"context"
 	"crypto/sha256"
 	"encoding/base64"
-	"encoding/binary"
 	"encoding/hex"
 	"errors"
 	"sort"
@@ -28,25 +27,21 @@ import (
 	"sync/atomic"
 	"time"
 
-	"go.etcd.io/etcd/api/v3/authpb"
-	pb "go.etcd.io/etcd/api/v3/etcdserverpb"
-	"go.etcd.io/etcd/api/v3/v3rpc/rpctypes"
-	"go.etcd.io/etcd/server/v3/mvcc/backend"
-	"go.etcd.io/etcd/server/v3/mvcc/buckets"
-
 	"go.uber.org/zap"
 	"golang.org/x/crypto/bcrypt"
 	"google.golang.org/grpc/credentials"
 	"google.golang.org/grpc/metadata"
 	"google.golang.org/grpc/peer"
+
+	"go.etcd.io/etcd/api/v3/authpb"
+	pb "go.etcd.io/etcd/api/v3/etcdserverpb"
+	"go.etcd.io/etcd/api/v3/v3rpc/rpctypes"
 )
 
-var (
-	enableFlagKey = []byte("authEnabled")
-	authEnabled   = []byte{1}
-	authDisabled  = []byte{0}
+var _ AuthStore = (*authStore)(nil)
 
-	revisionKey = []byte("authRevision")
+var (
+	rootPerm = authpb.Permission{PermType: authpb.READWRITE, Key: []byte{}, RangeEnd: []byte{0}}
 
 	ErrRootUserNotExist     = errors.New("auth: root user does not exist")
 	ErrRootRoleNotExist     = errors.New("auth: root user does not have root role")
@@ -79,8 +74,6 @@ const (
 
 	tokenTypeSimple = "simple"
 	tokenTypeJWT    = "jwt"
-
-	revBytesLen = 8
 )
 
 type AuthInfo struct {
@@ -109,7 +102,7 @@ type AuthStore interface {
 	Authenticate(ctx context.Context, username, password string) (*pb.AuthenticateResponse, error)
 
 	// Recover recovers the state of auth store from the given backend
-	Recover(b backend.Backend)
+	Recover(be AuthBackend)
 
 	// UserAdd adds a new user
 	UserAdd(r *pb.AuthUserAddRequest) (*pb.AuthUserAddResponse, error)
@@ -201,12 +194,59 @@ type TokenProvider interface {
 	genTokenPrefix() (string, error)
 }
 
+type AuthBackend interface {
+	CreateAuthBuckets()
+	ForceCommit()
+	ReadTx() AuthReadTx
+	BatchTx() AuthBatchTx
+
+	GetUser(string) *authpb.User
+	GetAllUsers() []*authpb.User
+	GetRole(string) *authpb.Role
+	GetAllRoles() []*authpb.Role
+}
+
+type AuthReadTx interface {
+	RLock()
+	RUnlock()
+	UnsafeAuthReader
+}
+
+type UnsafeAuthReader interface {
+	UnsafeReadAuthEnabled() bool
+	UnsafeReadAuthRevision() uint64
+	UnsafeGetUser(string) *authpb.User
+	UnsafeGetRole(string) *authpb.Role
+	UnsafeGetAllUsers() []*authpb.User
+	UnsafeGetAllRoles() []*authpb.Role
+}
+
+type AuthBatchTx interface {
+	Lock()
+	Unlock()
+	UnsafeAuthReadWriter
+}
+
+type UnsafeAuthReadWriter interface {
+	UnsafeAuthReader
+	UnsafeAuthWriter
+}
+
+type UnsafeAuthWriter interface {
+	UnsafeSaveAuthEnabled(enabled bool)
+	UnsafeSaveAuthRevision(rev uint64)
+	UnsafePutUser(*authpb.User)
+	UnsafeDeleteUser(string)
+	UnsafePutRole(*authpb.Role)
+	UnsafeDeleteRole(string)
+}
+
 type authStore struct {
 	// atomic operations; need 64-bit align, or 32-bit tests will crash
 	revision uint64
 
 	lg        *zap.Logger
-	be        backend.Backend
+	be        AuthBackend
 	enabled   bool
 	enabledMu sync.RWMutex
 
@@ -230,15 +270,14 @@ func (as *authStore) AuthEnable() error {
 		as.lg.Info("authentication is already enabled; ignored auth enable request")
 		return nil
 	}
-	b := as.be
-	tx := b.BatchTx()
-	tx.LockInsideApply()
+	tx := as.be.BatchTx()
+	tx.Lock()
 	defer func() {
 		tx.Unlock()
-		b.ForceCommit()
+		as.be.ForceCommit()
 	}()
 
-	u := getUser(as.lg, tx, rootUser)
+	u := tx.UnsafeGetUser(rootUser)
 	if u == nil {
 		return ErrRootUserNotExist
 	}
@@ -247,14 +286,13 @@ func (as *authStore) AuthEnable() error {
 		return ErrRootRoleNotExist
 	}
 
-	tx.UnsafePut(buckets.Auth, enableFlagKey, authEnabled)
-
+	tx.UnsafeSaveAuthEnabled(true)
 	as.enabled = true
 	as.tokenProvider.enable()
 
 	as.refreshRangePermCache(tx)
 
-	as.setRevision(getRevision(tx))
+	as.setRevision(tx.UnsafeReadAuthRevision())
 
 	as.lg.Info("enabled authentication")
 	return nil
@@ -267,11 +305,13 @@ func (as *authStore) AuthDisable() {
 		return
 	}
 	b := as.be
+
 	tx := b.BatchTx()
-	tx.LockInsideApply()
-	tx.UnsafePut(buckets.Auth, enableFlagKey, authDisabled)
+	tx.Lock()
+	tx.UnsafeSaveAuthEnabled(false)
 	as.commitRevision(tx)
 	tx.Unlock()
+
 	b.ForceCommit()
 
 	as.enabled = false
@@ -294,12 +334,7 @@ func (as *authStore) Authenticate(ctx context.Context, username, password string
 	if !as.IsAuthEnabled() {
 		return nil, ErrAuthNotEnabled
 	}
-
-	tx := as.be.BatchTx()
-	tx.LockInsideApply()
-	defer tx.Unlock()
-
-	user := getUser(as.lg, tx, username)
+	user := as.be.GetUser(username)
 	if user == nil {
 		return nil, ErrAuthFailed
 	}
@@ -333,10 +368,10 @@ func (as *authStore) CheckPassword(username, password string) (uint64, error) {
 	// to avoid putting it in the critical section of the tx lock.
 	revision, err := func() (uint64, error) {
 		tx := as.be.ReadTx()
-		tx.Lock()
-		defer tx.Unlock()
+		tx.RLock()
+		defer tx.RUnlock()
 
-		user = getUser(as.lg, tx, username)
+		user = tx.UnsafeGetUser(username)
 		if user == nil {
 			return 0, ErrAuthFailed
 		}
@@ -345,7 +380,7 @@ func (as *authStore) CheckPassword(username, password string) (uint64, error) {
 			return 0, ErrNoPasswordUser
 		}
 
-		return getRevision(tx), nil
+		return tx.UnsafeReadAuthRevision(), nil
 	}()
 	if err != nil {
 		return 0, err
@@ -358,22 +393,16 @@ func (as *authStore) CheckPassword(username, password string) (uint64, error) {
 	return revision, nil
 }
 
-func (as *authStore) Recover(be backend.Backend) {
-	enabled := false
+func (as *authStore) Recover(be AuthBackend) {
 	as.be = be
 	tx := be.ReadTx()
-	tx.Lock()
-	_, vs := tx.UnsafeRange(buckets.Auth, enableFlagKey, nil, 0)
-	if len(vs) == 1 {
-		if bytes.Equal(vs[0], authEnabled) {
-			enabled = true
-		}
-	}
+	tx.RLock()
 
-	as.setRevision(getRevision(tx))
+	enabled := tx.UnsafeReadAuthEnabled()
+	as.setRevision(tx.UnsafeReadAuthRevision())
 	as.refreshRangePermCache(tx)
 
-	tx.Unlock()
+	tx.RUnlock()
 
 	as.enabledMu.Lock()
 	as.enabled = enabled
@@ -397,10 +426,10 @@ func (as *authStore) UserAdd(r *pb.AuthUserAddRequest) (*pb.AuthUserAddResponse,
 	}
 
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	user := getUser(as.lg, tx, r.Name)
+	user := tx.UnsafeGetUser(r.Name)
 	if user != nil {
 		return nil, ErrUserAlreadyExist
 	}
@@ -427,8 +456,7 @@ func (as *authStore) UserAdd(r *pb.AuthUserAddRequest) (*pb.AuthUserAddResponse,
 		Password: password,
 		Options:  options,
 	}
-
-	putUser(as.lg, tx, newUser)
+	tx.UnsafePutUser(newUser)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -444,15 +472,14 @@ func (as *authStore) UserDelete(r *pb.AuthUserDeleteRequest) (*pb.AuthUserDelete
 	}
 
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	user := getUser(as.lg, tx, r.Name)
+	user := tx.UnsafeGetUser(r.Name)
 	if user == nil {
 		return nil, ErrUserNotFound
 	}
-
-	delUser(tx, r.Name)
+	tx.UnsafeDeleteUser(r.Name)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -469,10 +496,10 @@ func (as *authStore) UserDelete(r *pb.AuthUserDeleteRequest) (*pb.AuthUserDelete
 
 func (as *authStore) UserChangePassword(r *pb.AuthUserChangePasswordRequest) (*pb.AuthUserChangePasswordResponse, error) {
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	user := getUser(as.lg, tx, r.Name)
+	user := tx.UnsafeGetUser(r.Name)
 	if user == nil {
 		return nil, ErrUserNotFound
 	}
@@ -494,8 +521,7 @@ func (as *authStore) UserChangePassword(r *pb.AuthUserChangePasswordRequest) (*p
 		Password: password,
 		Options:  user.Options,
 	}
-
-	putUser(as.lg, tx, updatedUser)
+	tx.UnsafePutUser(updatedUser)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -512,16 +538,16 @@ func (as *authStore) UserChangePassword(r *pb.AuthUserChangePasswordRequest) (*p
 
 func (as *authStore) UserGrantRole(r *pb.AuthUserGrantRoleRequest) (*pb.AuthUserGrantRoleResponse, error) {
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	user := getUser(as.lg, tx, r.User)
+	user := tx.UnsafeGetUser(r.User)
 	if user == nil {
 		return nil, ErrUserNotFound
 	}
 
 	if r.Role != rootRole {
-		role := getRole(as.lg, tx, r.Role)
+		role := tx.UnsafeGetRole(r.Role)
 		if role == nil {
 			return nil, ErrRoleNotFound
 		}
@@ -541,7 +567,7 @@ func (as *authStore) UserGrantRole(r *pb.AuthUserGrantRoleRequest) (*pb.AuthUser
 	user.Roles = append(user.Roles, r.Role)
 	sort.Strings(user.Roles)
 
-	putUser(as.lg, tx, user)
+	tx.UnsafePutUser(user)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -556,10 +582,7 @@ func (as *authStore) UserGrantRole(r *pb.AuthUserGrantRoleRequest) (*pb.AuthUser
 }
 
 func (as *authStore) UserGet(r *pb.AuthUserGetRequest) (*pb.AuthUserGetResponse, error) {
-	tx := as.be.BatchTx()
-	tx.LockInsideApply()
-	user := getUser(as.lg, tx, r.Name)
-	tx.Unlock()
+	user := as.be.GetUser(r.Name)
 
 	if user == nil {
 		return nil, ErrUserNotFound
@@ -571,10 +594,7 @@ func (as *authStore) UserGet(r *pb.AuthUserGetRequest) (*pb.AuthUserGetResponse,
 }
 
 func (as *authStore) UserList(r *pb.AuthUserListRequest) (*pb.AuthUserListResponse, error) {
-	tx := as.be.BatchTx()
-	tx.LockInsideApply()
-	users := getAllUsers(as.lg, tx)
-	tx.Unlock()
+	users := as.be.GetAllUsers()
 
 	resp := &pb.AuthUserListResponse{Users: make([]string, len(users))}
 	for i := range users {
@@ -594,10 +614,10 @@ func (as *authStore) UserRevokeRole(r *pb.AuthUserRevokeRoleRequest) (*pb.AuthUs
 	}
 
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	user := getUser(as.lg, tx, r.Name)
+	user := tx.UnsafeGetUser(r.Name)
 	if user == nil {
 		return nil, ErrUserNotFound
 	}
@@ -618,7 +638,7 @@ func (as *authStore) UserRevokeRole(r *pb.AuthUserRevokeRoleRequest) (*pb.AuthUs
 		return nil, ErrRoleNotGranted
 	}
 
-	putUser(as.lg, tx, updatedUser)
+	tx.UnsafePutUser(updatedUser)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -634,25 +654,22 @@ func (as *authStore) UserRevokeRole(r *pb.AuthUserRevokeRoleRequest) (*pb.AuthUs
 }
 
 func (as *authStore) RoleGet(r *pb.AuthRoleGetRequest) (*pb.AuthRoleGetResponse, error) {
-	tx := as.be.BatchTx()
-	tx.LockInsideApply()
-	defer tx.Unlock()
-
 	var resp pb.AuthRoleGetResponse
 
-	role := getRole(as.lg, tx, r.Role)
+	role := as.be.GetRole(r.Role)
 	if role == nil {
 		return nil, ErrRoleNotFound
 	}
-	resp.Perm = append(resp.Perm, role.KeyPermission...)
+	if rootRole == string(role.Name) {
+		resp.Perm = append(resp.Perm, &rootPerm)
+	} else {
+		resp.Perm = append(resp.Perm, role.KeyPermission...)
+	}
 	return &resp, nil
 }
 
 func (as *authStore) RoleList(r *pb.AuthRoleListRequest) (*pb.AuthRoleListResponse, error) {
-	tx := as.be.BatchTx()
-	tx.LockInsideApply()
-	roles := getAllRoles(as.lg, tx)
-	tx.Unlock()
+	roles := as.be.GetAllRoles()
 
 	resp := &pb.AuthRoleListResponse{Roles: make([]string, len(roles))}
 	for i := range roles {
@@ -663,10 +680,10 @@ func (as *authStore) RoleList(r *pb.AuthRoleListRequest) (*pb.AuthRoleListRespon
 
 func (as *authStore) RoleRevokePermission(r *pb.AuthRoleRevokePermissionRequest) (*pb.AuthRoleRevokePermissionResponse, error) {
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	role := getRole(as.lg, tx, r.Role)
+	role := tx.UnsafeGetRole(r.Role)
 	if role == nil {
 		return nil, ErrRoleNotFound
 	}
@@ -685,7 +702,7 @@ func (as *authStore) RoleRevokePermission(r *pb.AuthRoleRevokePermissionRequest)
 		return nil, ErrPermissionNotGranted
 	}
 
-	putRole(as.lg, tx, updatedRole)
+	tx.UnsafePutRole(updatedRole)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -706,17 +723,17 @@ func (as *authStore) RoleDelete(r *pb.AuthRoleDeleteRequest) (*pb.AuthRoleDelete
 	}
 
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	role := getRole(as.lg, tx, r.Role)
+	role := tx.UnsafeGetRole(r.Role)
 	if role == nil {
 		return nil, ErrRoleNotFound
 	}
 
-	delRole(tx, r.Role)
+	tx.UnsafeDeleteRole(r.Role)
 
-	users := getAllUsers(as.lg, tx)
+	users := tx.UnsafeGetAllUsers()
 	for _, user := range users {
 		updatedUser := &authpb.User{
 			Name:     user.Name,
@@ -734,8 +751,7 @@ func (as *authStore) RoleDelete(r *pb.AuthRoleDeleteRequest) (*pb.AuthRoleDelete
 			continue
 		}
 
-		putUser(as.lg, tx, updatedUser)
-
+		tx.UnsafePutUser(updatedUser)
 	}
 
 	as.commitRevision(tx)
@@ -751,10 +767,10 @@ func (as *authStore) RoleAdd(r *pb.AuthRoleAddRequest) (*pb.AuthRoleAddResponse,
 	}
 
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	role := getRole(as.lg, tx, r.Name)
+	role := tx.UnsafeGetRole(r.Name)
 	if role != nil {
 		return nil, ErrRoleAlreadyExist
 	}
@@ -763,7 +779,7 @@ func (as *authStore) RoleAdd(r *pb.AuthRoleAddRequest) (*pb.AuthRoleAddResponse,
 		Name: []byte(r.Name),
 	}
 
-	putRole(as.lg, tx, newRole)
+	tx.UnsafePutRole(newRole)
 
 	as.commitRevision(tx)
 
@@ -798,10 +814,10 @@ func (as *authStore) RoleGrantPermission(r *pb.AuthRoleGrantPermissionRequest) (
 	}
 
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
+	tx.Lock()
 	defer tx.Unlock()
 
-	role := getRole(as.lg, tx, r.Name)
+	role := tx.UnsafeGetRole(r.Name)
 	if role == nil {
 		return nil, ErrRoleNotFound
 	}
@@ -825,7 +841,7 @@ func (as *authStore) RoleGrantPermission(r *pb.AuthRoleGrantPermissionRequest) (
 		sort.Sort(permSlice(role.KeyPermission))
 	}
 
-	putRole(as.lg, tx, role)
+	tx.UnsafePutRole(role)
 
 	as.commitRevision(tx)
 	as.refreshRangePermCache(tx)
@@ -834,6 +850,8 @@ func (as *authStore) RoleGrantPermission(r *pb.AuthRoleGrantPermissionRequest) (
 		"granted/updated a permission to a user",
 		zap.String("user-name", r.Name),
 		zap.String("permission-name", authpb.Permission_Type_name[int32(r.Perm.PermType)]),
+		zap.ByteString("key", r.Perm.Key),
+		zap.ByteString("range-end", r.Perm.RangeEnd),
 	)
 	return &pb.AuthRoleGrantPermissionResponse{}, nil
 }
@@ -859,10 +877,10 @@ func (as *authStore) isOpPermitted(userName string, revision uint64, key, rangeE
 	}
 
 	tx := as.be.ReadTx()
-	tx.Lock()
-	defer tx.Unlock()
+	tx.RLock()
+	defer tx.RUnlock()
 
-	user := getUser(as.lg, tx, userName)
+	user := tx.UnsafeGetUser(userName)
 	if user == nil {
 		as.lg.Error("cannot find a user for permission check", zap.String("user-name", userName))
 		return ErrPermissionDenied
@@ -901,9 +919,9 @@ func (as *authStore) IsAdminPermitted(authInfo *AuthInfo) error {
 	}
 
 	tx := as.be.ReadTx()
-	tx.Lock()
-	u := getUser(as.lg, tx, authInfo.Username)
-	tx.Unlock()
+	tx.RLock()
+	defer tx.RUnlock()
+	u := tx.UnsafeGetUser(authInfo.Username)
 
 	if u == nil {
 		return ErrUserNotFound
@@ -916,111 +934,6 @@ func (as *authStore) IsAdminPermitted(authInfo *AuthInfo) error {
 	return nil
 }
 
-func getUser(lg *zap.Logger, tx backend.ReadTx, username string) *authpb.User {
-	_, vs := tx.UnsafeRange(buckets.AuthUsers, []byte(username), nil, 0)
-	if len(vs) == 0 {
-		return nil
-	}
-
-	user := &authpb.User{}
-	err := user.Unmarshal(vs[0])
-	if err != nil {
-		lg.Panic(
-			"failed to unmarshal 'authpb.User'",
-			zap.String("user-name", username),
-			zap.Error(err),
-		)
-	}
-	return user
-}
-
-func getAllUsers(lg *zap.Logger, tx backend.ReadTx) []*authpb.User {
-	var vs [][]byte
-	err := tx.UnsafeForEach(buckets.AuthUsers, func(k []byte, v []byte) error {
-		vs = append(vs, v)
-		return nil
-	})
-	if err != nil {
-		lg.Panic("failed to get users",
-			zap.Error(err))
-	}
-	if len(vs) == 0 {
-		return nil
-	}
-
-	users := make([]*authpb.User, len(vs))
-	for i := range vs {
-		user := &authpb.User{}
-		err := user.Unmarshal(vs[i])
-		if err != nil {
-			lg.Panic("failed to unmarshal 'authpb.User'", zap.Error(err))
-		}
-		users[i] = user
-	}
-	return users
-}
-
-func putUser(lg *zap.Logger, tx backend.BatchTx, user *authpb.User) {
-	b, err := user.Marshal()
-	if err != nil {
-		lg.Panic("failed to unmarshal 'authpb.User'", zap.Error(err))
-	}
-	tx.UnsafePut(buckets.AuthUsers, user.Name, b)
-}
-
-func delUser(tx backend.BatchTx, username string) {
-	tx.UnsafeDelete(buckets.AuthUsers, []byte(username))
-}
-
-func getRole(lg *zap.Logger, tx backend.ReadTx, rolename string) *authpb.Role {
-	_, vs := tx.UnsafeRange(buckets.AuthRoles, []byte(rolename), nil, 0)
-	if len(vs) == 0 {
-		return nil
-	}
-
-	role := &authpb.Role{}
-	err := role.Unmarshal(vs[0])
-	if err != nil {
-		lg.Panic("failed to unmarshal 'authpb.Role'", zap.Error(err))
-	}
-	return role
-}
-
-func getAllRoles(lg *zap.Logger, tx backend.ReadTx) []*authpb.Role {
-	_, vs := tx.UnsafeRange(buckets.AuthRoles, []byte{0}, []byte{0xff}, -1)
-	if len(vs) == 0 {
-		return nil
-	}
-
-	roles := make([]*authpb.Role, len(vs))
-	for i := range vs {
-		role := &authpb.Role{}
-		err := role.Unmarshal(vs[i])
-		if err != nil {
-			lg.Panic("failed to unmarshal 'authpb.Role'", zap.Error(err))
-		}
-		roles[i] = role
-	}
-	return roles
-}
-
-func putRole(lg *zap.Logger, tx backend.BatchTx, role *authpb.Role) {
-	b, err := role.Marshal()
-	if err != nil {
-		lg.Panic(
-			"failed to marshal 'authpb.Role'",
-			zap.String("role-name", string(role.Name)),
-			zap.Error(err),
-		)
-	}
-
-	tx.UnsafePut(buckets.AuthRoles, role.Name, b)
-}
-
-func delRole(tx backend.BatchTx, rolename string) {
-	tx.UnsafeDelete(buckets.AuthRoles, []byte(rolename))
-}
-
 func (as *authStore) IsAuthEnabled() bool {
 	as.enabledMu.RLock()
 	defer as.enabledMu.RUnlock()
@@ -1028,7 +941,7 @@ func (as *authStore) IsAuthEnabled() bool {
 }
 
 // NewAuthStore creates a new AuthStore.
-func NewAuthStore(lg *zap.Logger, be backend.Backend, tp TokenProvider, bcryptCost int) *authStore {
+func NewAuthStore(lg *zap.Logger, be AuthBackend, tp TokenProvider, bcryptCost int) AuthStore {
 	if lg == nil {
 		lg = zap.NewNop()
 	}
@@ -1044,23 +957,14 @@ func NewAuthStore(lg *zap.Logger, be backend.Backend, tp TokenProvider, bcryptCo
 		bcryptCost = bcrypt.DefaultCost
 	}
 
+	be.CreateAuthBuckets()
 	tx := be.BatchTx()
-	tx.LockOutsideApply()
-
-	tx.UnsafeCreateBucket(buckets.Auth)
-	tx.UnsafeCreateBucket(buckets.AuthUsers)
-	tx.UnsafeCreateBucket(buckets.AuthRoles)
-
-	enabled := false
-	_, vs := tx.UnsafeRange(buckets.Auth, enableFlagKey, nil, 0)
-	if len(vs) == 1 {
-		if bytes.Equal(vs[0], authEnabled) {
-			enabled = true
-		}
-	}
-
+	// We should call LockOutsideApply here, but the txPostLockHoos isn't set
+	// to EtcdServer yet, so it's OK.
+	tx.Lock()
+	enabled := tx.UnsafeReadAuthEnabled()
 	as := &authStore{
-		revision:       getRevision(tx),
+		revision:       tx.UnsafeReadAuthRevision(),
 		lg:             lg,
 		be:             be,
 		enabled:        enabled,
@@ -1093,20 +997,9 @@ func hasRootRole(u *authpb.User) bool {
 	return idx != len(u.Roles) && u.Roles[idx] == rootRole
 }
 
-func (as *authStore) commitRevision(tx backend.BatchTx) {
+func (as *authStore) commitRevision(tx UnsafeAuthWriter) {
 	atomic.AddUint64(&as.revision, 1)
-	revBytes := make([]byte, revBytesLen)
-	binary.BigEndian.PutUint64(revBytes, as.Revision())
-	tx.UnsafePut(buckets.Auth, revisionKey, revBytes)
-}
-
-func getRevision(tx backend.ReadTx) uint64 {
-	_, vs := tx.UnsafeRange(buckets.Auth, revisionKey, nil, 0)
-	if len(vs) != 1 {
-		// this can happen in the initialization phase
-		return 0
-	}
-	return binary.BigEndian.Uint64(vs[0])
+	tx.UnsafeSaveAuthRevision(as.Revision())
 }
 
 func (as *authStore) setRevision(rev uint64) {
@@ -1164,12 +1057,13 @@ func (as *authStore) AuthInfoFromCtx(ctx context.Context) (*AuthInfo, error) {
 	if !as.IsAuthEnabled() {
 		return nil, nil
 	}
+
 	md, ok := metadata.FromIncomingContext(ctx)
 	if !ok {
 		return nil, nil
 	}
 
-	//TODO(mitake|hexfusion) review unifying key names
+	// TODO(mitake|hexfusion) review unifying key names
 	ts, ok := md[rpctypes.TokenFieldNameGRPC]
 	if !ok {
 		ts, ok = md[rpctypes.TokenFieldNameSwagger]
@@ -1223,7 +1117,6 @@ func decomposeOpts(lg *zap.Logger, optstr string) (string, map[string]string, er
 	}
 
 	return tokenType, typeSpecificOpts, nil
-
 }
 
 // NewTokenProvider creates a new token provider.
@@ -1231,7 +1124,8 @@ func NewTokenProvider(
 	lg *zap.Logger,
 	tokenOpts string,
 	indexWaiter func(uint64) <-chan struct{},
-	TokenTTL time.Duration) (TokenProvider, error) {
+	TokenTTL time.Duration,
+) (TokenProvider, error) {
 	tokenType, typeSpecificOpts, err := decomposeOpts(lg, tokenOpts)
 	if err != nil {
 		return nil, ErrInvalidAuthOpts
@@ -1304,8 +1198,8 @@ func (as *authStore) WithRoot(ctx context.Context) context.Context {
 
 func (as *authStore) HasRole(user, role string) bool {
 	tx := as.be.BatchTx()
-	tx.LockInsideApply()
-	u := getUser(as.lg, tx, user)
+	tx.Lock()
+	u := tx.UnsafeGetUser(user)
 	tx.Unlock()
 
 	if u == nil {
