@@ -19,26 +19,25 @@ import (
 	"encoding/base64"
 	"errors"
 	"fmt"
-	"reflect"
 	"strings"
 	"sync"
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+	"go.uber.org/zap/zaptest"
+	"golang.org/x/crypto/bcrypt"
+	"google.golang.org/grpc/metadata"
+
 	"go.etcd.io/etcd/api/v3/authpb"
 	pb "go.etcd.io/etcd/api/v3/etcdserverpb"
 	"go.etcd.io/etcd/api/v3/v3rpc/rpctypes"
 	"go.etcd.io/etcd/pkg/v3/adt"
-	"go.etcd.io/etcd/server/v3/mvcc/backend"
-	betesting "go.etcd.io/etcd/server/v3/mvcc/backend/testing"
-
-	"go.uber.org/zap"
-	"golang.org/x/crypto/bcrypt"
-	"google.golang.org/grpc/metadata"
 )
 
 func dummyIndexWaiter(index uint64) <-chan struct{} {
-	ch := make(chan struct{})
+	ch := make(chan struct{}, 1)
 	go func() {
 		ch <- struct{}{}
 	}()
@@ -48,66 +47,53 @@ func dummyIndexWaiter(index uint64) <-chan struct{} {
 // TestNewAuthStoreRevision ensures newly auth store
 // keeps the old revision when there are no changes.
 func TestNewAuthStoreRevision(t *testing.T) {
-	b, tPath := betesting.NewDefaultTmpBackend(t)
-
-	tp, err := NewTokenProvider(zap.NewExample(), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
 	if err != nil {
 		t.Fatal(err)
 	}
-	as := NewAuthStore(zap.NewExample(), b, tp, bcrypt.MinCost)
+	be := newBackendMock()
+	as := NewAuthStore(zaptest.NewLogger(t), be, tp, bcrypt.MinCost)
 	err = enableAuthAndCreateRoot(as)
 	if err != nil {
 		t.Fatal(err)
 	}
 	old := as.Revision()
 	as.Close()
-	b.Close()
 
 	// no changes to commit
-	b2 := backend.NewDefaultBackend(tPath)
-	defer b2.Close()
-	as = NewAuthStore(zap.NewExample(), b2, tp, bcrypt.MinCost)
+	as = NewAuthStore(zaptest.NewLogger(t), be, tp, bcrypt.MinCost)
 	defer as.Close()
 	new := as.Revision()
 
-	if old != new {
-		t.Fatalf("expected revision %d, got %d", old, new)
-	}
+	require.Equalf(t, old, new, "expected revision %d, got %d", old, new)
 }
 
 // TestNewAuthStoreBcryptCost ensures that NewAuthStore uses default when given bcrypt-cost is invalid
 func TestNewAuthStoreBcryptCost(t *testing.T) {
-	b, _ := betesting.NewDefaultTmpBackend(t)
-	defer betesting.Close(t, b)
-
-	tp, err := NewTokenProvider(zap.NewExample(), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
 	if err != nil {
 		t.Fatal(err)
 	}
 
 	invalidCosts := [2]int{bcrypt.MinCost - 1, bcrypt.MaxCost + 1}
 	for _, invalidCost := range invalidCosts {
-		as := NewAuthStore(zap.NewExample(), b, tp, invalidCost)
+		as := NewAuthStore(zaptest.NewLogger(t), newBackendMock(), tp, invalidCost)
 		defer as.Close()
-		if as.BcryptCost() != bcrypt.DefaultCost {
-			t.Fatalf("expected DefaultCost when bcryptcost is invalid")
-		}
+		require.Equalf(t, bcrypt.DefaultCost, as.BcryptCost(), "expected DefaultCost when bcryptcost is invalid")
 	}
 }
 
 func encodePassword(s string) string {
 	hashedPassword, _ := bcrypt.GenerateFromPassword([]byte(s), bcrypt.MinCost)
-	return base64.StdEncoding.EncodeToString([]byte(hashedPassword))
+	return base64.StdEncoding.EncodeToString(hashedPassword)
 }
 
 func setupAuthStore(t *testing.T) (store *authStore, teardownfunc func(t *testing.T)) {
-	b, _ := betesting.NewDefaultTmpBackend(t)
-
-	tp, err := NewTokenProvider(zap.NewExample(), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
 	if err != nil {
 		t.Fatal(err)
 	}
-	as := NewAuthStore(zap.NewExample(), b, tp, bcrypt.MinCost)
+	as := NewAuthStore(zaptest.NewLogger(t), newBackendMock(), tp, bcrypt.MinCost)
 	err = enableAuthAndCreateRoot(as)
 	if err != nil {
 		t.Fatal(err)
@@ -127,27 +113,29 @@ func setupAuthStore(t *testing.T) (store *authStore, teardownfunc func(t *testin
 
 	// The UserAdd function cannot generate old etcd version user data (user's option is nil)
 	// add special users through the underlying interface
-	err = addUserWithNoOption(as)
-	if err != nil {
-		t.Fatal(err)
-	}
+	asImpl, ok := as.(*authStore)
+	require.Truef(t, ok, "addUserWithNoOption: needs an AuthStore implementation")
+	addUserWithNoOption(asImpl)
 
 	tearDown := func(_ *testing.T) {
-		b.Close()
 		as.Close()
 	}
-	return as, tearDown
+	return asImpl, tearDown
 }
 
-func addUserWithNoOption(as *authStore) error {
-	_, err := as.UserAdd(&pb.AuthUserAddRequest{Name: "foo-no-user-options", Password: "bar"})
-	if err != nil {
-		return err
-	}
-	return nil
+func addUserWithNoOption(as *authStore) {
+	tx := as.be.BatchTx()
+	tx.Lock()
+	defer tx.Unlock()
+	tx.UnsafePutUser(&authpb.User{
+		Name:     []byte("foo-no-user-options"),
+		Password: []byte("bar"),
+	})
+	as.commitRevision(tx)
+	as.refreshRangePermCache(tx)
 }
 
-func enableAuthAndCreateRoot(as *authStore) error {
+func enableAuthAndCreateRoot(as AuthStore) error {
 	_, err := as.UserAdd(&pb.AuthUserAddRequest{Name: "root", HashedPassword: encodePassword("root"), Options: &authpb.UserAddOptions{NoPassword: false}})
 	if err != nil {
 		return err
@@ -173,23 +161,17 @@ func TestUserAdd(t *testing.T) {
 	const userName = "foo"
 	ua := &pb.AuthUserAddRequest{Name: userName, Options: &authpb.UserAddOptions{NoPassword: false}}
 	_, err := as.UserAdd(ua) // add an existing user
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrUserAlreadyExist, err)
-	}
-	if err != ErrUserAlreadyExist {
-		t.Fatalf("expected %v, got %v", ErrUserAlreadyExist, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrUserAlreadyExist, err)
+	require.ErrorIsf(t, err, ErrUserAlreadyExist, "expected %v, got %v", ErrUserAlreadyExist, err)
 
 	ua = &pb.AuthUserAddRequest{Name: "", Options: &authpb.UserAddOptions{NoPassword: false}}
 	_, err = as.UserAdd(ua) // add a user with empty name
-	if err != ErrUserEmpty {
+	if !errors.Is(err, ErrUserEmpty) {
 		t.Fatal(err)
 	}
 
-	if _, ok := as.rangePermCache[userName]; !ok {
-		t.Fatalf("user %s should be added but it doesn't exist in rangePermCache", userName)
-
-	}
+	_, ok := as.rangePermCache[userName]
+	require.Truef(t, ok, "user %s should be added but it doesn't exist in rangePermCache", userName)
 }
 
 func TestRecover(t *testing.T) {
@@ -200,9 +182,7 @@ func TestRecover(t *testing.T) {
 	as.enabled = false
 	as.Recover(as.be)
 
-	if !as.IsAuthEnabled() {
-		t.Fatalf("expected auth enabled got disabled")
-	}
+	require.Truef(t, as.IsAuthEnabled(), "expected auth enabled got disabled")
 }
 
 func TestRecoverWithEmptyRangePermCache(t *testing.T) {
@@ -214,19 +194,13 @@ func TestRecoverWithEmptyRangePermCache(t *testing.T) {
 	as.rangePermCache = map[string]*unifiedRangePermissions{}
 	as.Recover(as.be)
 
-	if !as.IsAuthEnabled() {
-		t.Fatalf("expected auth enabled got disabled")
-	}
+	require.Truef(t, as.IsAuthEnabled(), "expected auth enabled got disabled")
 
-	if len(as.rangePermCache) != 3 {
-		t.Fatalf("rangePermCache should have permission information for 3 users (\"root\" and \"foo\",\"foo-no-user-options\"), but has %d information", len(as.rangePermCache))
-	}
-	if _, ok := as.rangePermCache["root"]; !ok {
-		t.Fatal("user \"root\" should be created by setupAuthStore() but doesn't exist in rangePermCache")
-	}
-	if _, ok := as.rangePermCache["foo"]; !ok {
-		t.Fatal("user \"foo\" should be created by setupAuthStore() but doesn't exist in rangePermCache")
-	}
+	require.Lenf(t, as.rangePermCache, 3, "rangePermCache should have permission information for 3 users (\"root\" and \"foo\",\"foo-no-user-options\"), but has %d information", len(as.rangePermCache))
+	_, ok := as.rangePermCache["root"]
+	require.Truef(t, ok, "user \"root\" should be created by setupAuthStore() but doesn't exist in rangePermCache")
+	_, ok = as.rangePermCache["foo"]
+	require.Truef(t, ok, "user \"foo\" should be created by setupAuthStore() but doesn't exist in rangePermCache")
 }
 
 func TestCheckPassword(t *testing.T) {
@@ -235,12 +209,8 @@ func TestCheckPassword(t *testing.T) {
 
 	// auth a non-existing user
 	_, err := as.CheckPassword("foo-test", "bar")
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrAuthFailed, err)
-	}
-	if err != ErrAuthFailed {
-		t.Fatalf("expected %v, got %v", ErrAuthFailed, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrAuthFailed, err)
+	require.ErrorIsf(t, err, ErrAuthFailed, "expected %v, got %v", ErrAuthFailed, err)
 
 	// auth an existing user with correct password
 	_, err = as.CheckPassword("foo", "bar")
@@ -250,12 +220,8 @@ func TestCheckPassword(t *testing.T) {
 
 	// auth an existing user but with wrong password
 	_, err = as.CheckPassword("foo", "")
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrAuthFailed, err)
-	}
-	if err != ErrAuthFailed {
-		t.Fatalf("expected %v, got %v", ErrAuthFailed, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrAuthFailed, err)
+	require.ErrorIsf(t, err, ErrAuthFailed, "expected %v, got %v", ErrAuthFailed, err)
 }
 
 func TestUserDelete(t *testing.T) {
@@ -272,17 +238,11 @@ func TestUserDelete(t *testing.T) {
 
 	// delete a non-existing user
 	_, err = as.UserDelete(ud)
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
-	if err != ErrUserNotFound {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrUserNotFound, err)
+	require.ErrorIsf(t, err, ErrUserNotFound, "expected %v, got %v", ErrUserNotFound, err)
 
-	if _, ok := as.rangePermCache[userName]; ok {
-		t.Fatalf("user %s should be deleted but it exists in rangePermCache", userName)
-
-	}
+	_, ok := as.rangePermCache[userName]
+	require.Falsef(t, ok, "user %s should be deleted but it exists in rangePermCache", userName)
 }
 
 func TestUserDeleteAndPermCache(t *testing.T) {
@@ -299,13 +259,10 @@ func TestUserDeleteAndPermCache(t *testing.T) {
 
 	// delete a non-existing user
 	_, err = as.UserDelete(ud)
-	if err != ErrUserNotFound {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
+	require.ErrorIsf(t, err, ErrUserNotFound, "expected %v, got %v", ErrUserNotFound, err)
 
-	if _, ok := as.rangePermCache[deletedUserName]; ok {
-		t.Fatalf("user %s should be deleted but it exists in rangePermCache", deletedUserName)
-	}
+	_, ok := as.rangePermCache[deletedUserName]
+	require.Falsef(t, ok, "user %s should be deleted but it exists in rangePermCache", deletedUserName)
 
 	// add a new user
 	const newUser = "bar"
@@ -315,10 +272,8 @@ func TestUserDeleteAndPermCache(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	if _, ok := as.rangePermCache[newUser]; !ok {
-		t.Fatalf("user %s should exist but it doesn't exist in rangePermCache", deletedUserName)
-
-	}
+	_, ok = as.rangePermCache[newUser]
+	require.Truef(t, ok, "user %s should exist but it doesn't exist in rangePermCache", deletedUserName)
 }
 
 func TestUserChangePassword(t *testing.T) {
@@ -344,12 +299,8 @@ func TestUserChangePassword(t *testing.T) {
 
 	// change a non-existing user
 	_, err = as.UserChangePassword(&pb.AuthUserChangePasswordRequest{Name: "foo-test", HashedPassword: encodePassword("bar")})
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
-	if err != ErrUserNotFound {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrUserNotFound, err)
+	require.ErrorIsf(t, err, ErrUserNotFound, "expected %v, got %v", ErrUserNotFound, err)
 
 	// change a user（user option is nil) password
 	_, err = as.UserChangePassword(&pb.AuthUserChangePasswordRequest{Name: "foo-no-user-options", HashedPassword: encodePassword("bar")})
@@ -370,7 +321,7 @@ func TestRoleAdd(t *testing.T) {
 
 	// add a role with empty name
 	_, err = as.RoleAdd(&pb.AuthRoleAddRequest{Name: ""})
-	if err != ErrRoleEmpty {
+	if !errors.Is(err, ErrRoleEmpty) {
 		t.Fatal(err)
 	}
 }
@@ -390,7 +341,7 @@ func TestUserGrant(t *testing.T) {
 	if err == nil {
 		t.Errorf("expected %v, got %v", ErrUserNotFound, err)
 	}
-	if err != ErrUserNotFound {
+	if !errors.Is(err, ErrUserNotFound) {
 		t.Errorf("expected %v, got %v", ErrUserNotFound, err)
 	}
 }
@@ -407,21 +358,15 @@ func TestHasRole(t *testing.T) {
 
 	// checks role reflects correctly
 	hr := as.HasRole("foo", "role-test")
-	if !hr {
-		t.Fatal("expected role granted, got false")
-	}
+	require.Truef(t, hr, "expected role granted, got false")
 
 	// checks non existent role
 	hr = as.HasRole("foo", "non-existent-role")
-	if hr {
-		t.Fatal("expected role not found, got true")
-	}
+	require.Falsef(t, hr, "expected role not found, got true")
 
 	// checks non existent user
 	hr = as.HasRole("nouser", "role-test")
-	if hr {
-		t.Fatal("expected user not found got true")
-	}
+	require.Falsef(t, hr, "expected user not found got true")
 }
 
 func TestIsOpPermitted(t *testing.T) {
@@ -460,6 +405,15 @@ func TestIsOpPermitted(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
+
+	// Drop the user's permission from cache and expect a permission denied
+	// error.
+	as.rangePermCacheMu.Lock()
+	delete(as.rangePermCache, "foo")
+	as.rangePermCacheMu.Unlock()
+	if err := as.isOpPermitted("foo", as.Revision(), perm.Key, perm.RangeEnd, perm.PermType); !errors.Is(err, ErrPermissionDenied) {
+		t.Fatal(err)
+	}
 }
 
 func TestGetUser(t *testing.T) {
@@ -475,13 +429,10 @@ func TestGetUser(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	if u == nil {
-		t.Fatal("expect user not nil, got nil")
-	}
+	require.NotNilf(t, u, "expect user not nil, got nil")
 	expected := []string{"role-test"}
-	if !reflect.DeepEqual(expected, u.Roles) {
-		t.Errorf("expected %v, got %v", expected, u.Roles)
-	}
+
+	assert.Equal(t, expected, u.Roles)
 
 	// check non existent user
 	_, err = as.UserGet(&pb.AuthUserGetRequest{Name: "nouser"})
@@ -530,7 +481,6 @@ func TestRoleGrantPermission(t *testing.T) {
 		Name: "role-test-1",
 		Perm: perm,
 	})
-
 	if err != nil {
 		t.Error(err)
 	}
@@ -540,16 +490,14 @@ func TestRoleGrantPermission(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	if !reflect.DeepEqual(perm, r.Perm[0]) {
-		t.Errorf("expected %v, got %v", perm, r.Perm[0])
-	}
+	assert.Equal(t, perm, r.Perm[0])
 
 	// trying to grant nil permissions returns an error (and doesn't change the actual permissions!)
 	_, err = as.RoleGrantPermission(&pb.AuthRoleGrantPermissionRequest{
 		Name: "role-test-1",
 	})
 
-	if err != ErrPermissionNotGiven {
+	if !errors.Is(err, ErrPermissionNotGiven) {
 		t.Error(err)
 	}
 
@@ -558,9 +506,7 @@ func TestRoleGrantPermission(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	if !reflect.DeepEqual(perm, r.Perm[0]) {
-		t.Errorf("expected %v, got %v", perm, r.Perm[0])
-	}
+	assert.Equal(t, perm, r.Perm[0])
 }
 
 func TestRoleGrantInvalidPermission(t *testing.T) {
@@ -701,6 +647,38 @@ func TestRoleGrantInvalidPermission(t *testing.T) {
 	}
 }
 
+func TestRootRoleGrantPermission(t *testing.T) {
+	as, tearDown := setupAuthStore(t)
+	defer tearDown(t)
+
+	perm := &authpb.Permission{
+		PermType: authpb.WRITE,
+		Key:      []byte("Keys"),
+		RangeEnd: []byte("RangeEnd"),
+	}
+	_, err := as.RoleGrantPermission(&pb.AuthRoleGrantPermissionRequest{
+		Name: "root",
+		Perm: perm,
+	})
+	if err != nil {
+		t.Error(err)
+	}
+
+	r, err := as.RoleGet(&pb.AuthRoleGetRequest{Role: "root"})
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// whatever grant permission to root, it always return root permission.
+	expectPerm := &authpb.Permission{
+		PermType: authpb.READWRITE,
+		Key:      []byte{},
+		RangeEnd: []byte{0},
+	}
+
+	assert.Equal(t, expectPerm, r.Perm[0])
+}
+
 func TestRoleRevokePermission(t *testing.T) {
 	as, tearDown := setupAuthStore(t)
 	defer tearDown(t)
@@ -719,7 +697,6 @@ func TestRoleRevokePermission(t *testing.T) {
 		Name: "role-test-1",
 		Perm: perm,
 	})
-
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -781,18 +758,13 @@ func TestUserRevokePermission(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	if _, ok := as.rangePermCache[userName]; !ok {
-		t.Fatalf("User %s should have its entry in rangePermCache", userName)
-	}
+	_, ok := as.rangePermCache[userName]
+	require.Truef(t, ok, "User %s should have its entry in rangePermCache", userName)
 	unifiedPerm := as.rangePermCache[userName]
 	pt1 := adt.NewBytesAffinePoint([]byte("WriteKeyBegin"))
-	if !unifiedPerm.writePerms.Contains(pt1) {
-		t.Fatal("rangePermCache should contain WriteKeyBegin")
-	}
+	require.Truef(t, unifiedPerm.writePerms.Contains(pt1), "rangePermCache should contain WriteKeyBegin")
 	pt2 := adt.NewBytesAffinePoint([]byte("OutOfRange"))
-	if unifiedPerm.writePerms.Contains(pt2) {
-		t.Fatal("rangePermCache should not contain OutOfRange")
-	}
+	require.Falsef(t, unifiedPerm.writePerms.Contains(pt2), "rangePermCache should not contain OutOfRange")
 
 	u, err := as.UserGet(&pb.AuthUserGetRequest{Name: userName})
 	if err != nil {
@@ -800,9 +772,8 @@ func TestUserRevokePermission(t *testing.T) {
 	}
 
 	expected := []string{"role-test", "role-test-1"}
-	if !reflect.DeepEqual(expected, u.Roles) {
-		t.Fatalf("expected %v, got %v", expected, u.Roles)
-	}
+
+	assert.Equal(t, expected, u.Roles)
 
 	_, err = as.UserRevokeRole(&pb.AuthUserRevokeRoleRequest{Name: userName, Role: "role-test-1"})
 	if err != nil {
@@ -815,9 +786,8 @@ func TestUserRevokePermission(t *testing.T) {
 	}
 
 	expected = []string{"role-test"}
-	if !reflect.DeepEqual(expected, u.Roles) {
-		t.Errorf("expected %v, got %v", expected, u.Roles)
-	}
+
+	assert.Equal(t, expected, u.Roles)
 }
 
 func TestRoleDelete(t *testing.T) {
@@ -833,9 +803,8 @@ func TestRoleDelete(t *testing.T) {
 		t.Fatal(err)
 	}
 	expected := []string{"root"}
-	if !reflect.DeepEqual(expected, rl.Roles) {
-		t.Errorf("expected %v, got %v", expected, rl.Roles)
-	}
+
+	assert.Equal(t, expected, rl.Roles)
 }
 
 func TestAuthInfoFromCtx(t *testing.T) {
@@ -863,13 +832,13 @@ func TestAuthInfoFromCtx(t *testing.T) {
 
 	ctx = metadata.NewIncomingContext(context.Background(), metadata.New(map[string]string{rpctypes.TokenFieldNameGRPC: "Invalid Token"}))
 	_, err = as.AuthInfoFromCtx(ctx)
-	if err != ErrInvalidAuthToken {
+	if !errors.Is(err, ErrInvalidAuthToken) {
 		t.Errorf("expected %v, got %v", ErrInvalidAuthToken, err)
 	}
 
 	ctx = metadata.NewIncomingContext(context.Background(), metadata.New(map[string]string{rpctypes.TokenFieldNameGRPC: "Invalid.Token"}))
 	_, err = as.AuthInfoFromCtx(ctx)
-	if err != ErrInvalidAuthToken {
+	if !errors.Is(err, ErrInvalidAuthToken) {
 		t.Errorf("expected %v, got %v", ErrInvalidAuthToken, err)
 	}
 
@@ -890,14 +859,14 @@ func TestAuthDisable(t *testing.T) {
 	as.AuthDisable()
 	ctx := context.WithValue(context.WithValue(context.TODO(), AuthenticateParamIndex{}, uint64(2)), AuthenticateParamSimpleTokenPrefix{}, "dummy")
 	_, err := as.Authenticate(ctx, "foo", "bar")
-	if err != ErrAuthNotEnabled {
+	if !errors.Is(err, ErrAuthNotEnabled) {
 		t.Errorf("expected %v, got %v", ErrAuthNotEnabled, err)
 	}
 
 	// Disabling disabled auth to make sure it can return safely if store is already disabled.
 	as.AuthDisable()
 	_, err = as.Authenticate(ctx, "foo", "bar")
-	if err != ErrAuthNotEnabled {
+	if !errors.Is(err, ErrAuthNotEnabled) {
 		t.Errorf("expected %v, got %v", ErrAuthNotEnabled, err)
 	}
 }
@@ -928,14 +897,11 @@ func TestIsAuthEnabled(t *testing.T) {
 
 // TestAuthInfoFromCtxRace ensures that access to authStore.revision is thread-safe.
 func TestAuthInfoFromCtxRace(t *testing.T) {
-	b, _ := betesting.NewDefaultTmpBackend(t)
-	defer betesting.Close(t, b)
-
-	tp, err := NewTokenProvider(zap.NewExample(), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
 	if err != nil {
 		t.Fatal(err)
 	}
-	as := NewAuthStore(zap.NewExample(), b, tp, bcrypt.MinCost)
+	as := NewAuthStore(zaptest.NewLogger(t), newBackendMock(), tp, bcrypt.MinCost)
 	defer as.Close()
 
 	donec := make(chan struct{})
@@ -959,19 +925,19 @@ func TestIsAdminPermitted(t *testing.T) {
 
 	// invalid user
 	err = as.IsAdminPermitted(&AuthInfo{Username: "rooti", Revision: 1})
-	if err != ErrUserNotFound {
+	if !errors.Is(err, ErrUserNotFound) {
 		t.Errorf("expected %v, got %v", ErrUserNotFound, err)
 	}
 
 	// empty user
 	err = as.IsAdminPermitted(&AuthInfo{Username: "", Revision: 1})
-	if err != ErrUserEmpty {
+	if !errors.Is(err, ErrUserEmpty) {
 		t.Errorf("expected %v, got %v", ErrUserEmpty, err)
 	}
 
 	// non-admin user
 	err = as.IsAdminPermitted(&AuthInfo{Username: "foo", Revision: 1})
-	if err != ErrPermissionDenied {
+	if !errors.Is(err, ErrPermissionDenied) {
 		t.Errorf("expected %v, got %v", ErrPermissionDenied, err)
 	}
 
@@ -989,31 +955,25 @@ func TestRecoverFromSnapshot(t *testing.T) {
 
 	ua := &pb.AuthUserAddRequest{Name: "foo", Options: &authpb.UserAddOptions{NoPassword: false}}
 	_, err := as.UserAdd(ua) // add an existing user
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrUserAlreadyExist, err)
-	}
-	if err != ErrUserAlreadyExist {
-		t.Fatalf("expected %v, got %v", ErrUserAlreadyExist, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrUserAlreadyExist, err)
+	require.ErrorIsf(t, err, ErrUserAlreadyExist, "expected %v, got %v", ErrUserAlreadyExist, err)
 
 	ua = &pb.AuthUserAddRequest{Name: "", Options: &authpb.UserAddOptions{NoPassword: false}}
 	_, err = as.UserAdd(ua) // add a user with empty name
-	if err != ErrUserEmpty {
+	if !errors.Is(err, ErrUserEmpty) {
 		t.Fatal(err)
 	}
 
 	as.Close()
 
-	tp, err := NewTokenProvider(zap.NewExample(), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
 	if err != nil {
 		t.Fatal(err)
 	}
-	as2 := NewAuthStore(zap.NewExample(), as.be, tp, bcrypt.MinCost)
+	as2 := NewAuthStore(zaptest.NewLogger(t), as.be, tp, bcrypt.MinCost)
 	defer as2.Close()
 
-	if !as2.IsAuthEnabled() {
-		t.Fatal("recovering authStore from existing backend failed")
-	}
+	require.Truef(t, as2.IsAuthEnabled(), "recovering authStore from existing backend failed")
 
 	ul, err := as.UserList(&pb.AuthUserListRequest{})
 	if err != nil {
@@ -1081,15 +1041,12 @@ func TestHammerSimpleAuthenticate(t *testing.T) {
 
 // TestRolesOrder tests authpb.User.Roles is sorted
 func TestRolesOrder(t *testing.T) {
-	b, _ := betesting.NewDefaultTmpBackend(t)
-	defer betesting.Close(t, b)
-
-	tp, err := NewTokenProvider(zap.NewExample(), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), tokenTypeSimple, dummyIndexWaiter, simpleTokenTTLDefault)
 	defer tp.disable()
 	if err != nil {
 		t.Fatal(err)
 	}
-	as := NewAuthStore(zap.NewExample(), b, tp, bcrypt.MinCost)
+	as := NewAuthStore(zaptest.NewLogger(t), newBackendMock(), tp, bcrypt.MinCost)
 	defer as.Close()
 	err = enableAuthAndCreateRoot(as)
 	if err != nil {
@@ -1138,14 +1095,11 @@ func TestAuthInfoFromCtxWithRootJWT(t *testing.T) {
 
 // testAuthInfoFromCtxWithRoot ensures "WithRoot" properly embeds token in the context.
 func testAuthInfoFromCtxWithRoot(t *testing.T, opts string) {
-	b, _ := betesting.NewDefaultTmpBackend(t)
-	defer betesting.Close(t, b)
-
-	tp, err := NewTokenProvider(zap.NewExample(), opts, dummyIndexWaiter, simpleTokenTTLDefault)
+	tp, err := NewTokenProvider(zaptest.NewLogger(t), opts, dummyIndexWaiter, simpleTokenTTLDefault)
 	if err != nil {
 		t.Fatal(err)
 	}
-	as := NewAuthStore(zap.NewExample(), b, tp, bcrypt.MinCost)
+	as := NewAuthStore(zaptest.NewLogger(t), newBackendMock(), tp, bcrypt.MinCost)
 	defer as.Close()
 
 	if err = enableAuthAndCreateRoot(as); err != nil {
@@ -1157,11 +1111,9 @@ func testAuthInfoFromCtxWithRoot(t *testing.T, opts string) {
 
 	ai, aerr := as.AuthInfoFromCtx(ctx)
 	if aerr != nil {
-		t.Error(err)
-	}
-	if ai == nil {
-		t.Error("expected non-nil *AuthInfo")
+		t.Fatal(err)
 	}
+	require.NotNilf(t, ai, "expected non-nil *AuthInfo")
 	if ai.Username != "root" {
 		t.Errorf("expected user name 'root', got %+v", ai)
 	}
@@ -1180,9 +1132,7 @@ func TestUserNoPasswordAdd(t *testing.T) {
 
 	ctx := context.WithValue(context.WithValue(context.TODO(), AuthenticateParamIndex{}, uint64(1)), AuthenticateParamSimpleTokenPrefix{}, "dummy")
 	_, err = as.Authenticate(ctx, username, "")
-	if err != ErrAuthFailed {
-		t.Fatalf("expected %v, got %v", ErrAuthFailed, err)
-	}
+	require.ErrorIsf(t, err, ErrAuthFailed, "expected %v, got %v", ErrAuthFailed, err)
 }
 
 func TestUserAddWithOldLog(t *testing.T) {
@@ -1219,10 +1169,6 @@ func TestUserChangePasswordWithOldLog(t *testing.T) {
 
 	// change a non-existing user
 	_, err = as.UserChangePassword(&pb.AuthUserChangePasswordRequest{Name: "foo-test", HashedPassword: encodePassword("bar")})
-	if err == nil {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
-	if err != ErrUserNotFound {
-		t.Fatalf("expected %v, got %v", ErrUserNotFound, err)
-	}
+	require.Errorf(t, err, "expected %v, got %v", ErrUserNotFound, err)
+	require.ErrorIsf(t, err, ErrUserNotFound, "expected %v, got %v", ErrUserNotFound, err)
 }
