@@ -15,18 +15,21 @@
 package e2e
 
 import (
+	"context"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"os"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/sync/errgroup"
 
 	"go.etcd.io/etcd/pkg/v3/expect"
 	"go.etcd.io/etcd/server/v3/embed"
+	"go.etcd.io/etcd/tests/v3/framework/config"
 	"go.etcd.io/etcd/tests/v3/framework/e2e"
 )
 
@@ -35,16 +38,10 @@ const exampleConfigFile = "../../etcd.conf.yml.sample"
 func TestEtcdExampleConfig(t *testing.T) {
 	e2e.SkipInShortMode(t)
 
-	proc, err := e2e.SpawnCmd([]string{e2e.BinDir + "/etcd", "--config-file", exampleConfigFile}, nil)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if err = e2e.WaitReadyExpectProc(proc, e2e.EtcdServerReadyLines); err != nil {
-		t.Fatal(err)
-	}
-	if err = proc.Stop(); err != nil {
-		t.Fatal(err)
-	}
+	proc, err := e2e.SpawnCmd([]string{e2e.BinPath.Etcd, "--config-file", exampleConfigFile}, nil)
+	require.NoError(t, err)
+	require.NoError(t, e2e.WaitReadyExpectProc(context.TODO(), proc, e2e.EtcdServerReadyLines))
+	require.NoError(t, proc.Stop())
 }
 
 func TestEtcdMultiPeer(t *testing.T) {
@@ -53,11 +50,7 @@ func TestEtcdMultiPeer(t *testing.T) {
 	peers, tmpdirs := make([]string, 3), make([]string, 3)
 	for i := range peers {
 		peers[i] = fmt.Sprintf("e%d=http://127.0.0.1:%d", i, e2e.EtcdProcessBasePort+i)
-		d, err := ioutil.TempDir("", fmt.Sprintf("e%d.etcd", i))
-		if err != nil {
-			t.Fatal(err)
-		}
-		tmpdirs[i] = d
+		tmpdirs[i] = t.TempDir()
 	}
 	ic := strings.Join(peers, ",")
 
@@ -66,13 +59,13 @@ func TestEtcdMultiPeer(t *testing.T) {
 		for i := range procs {
 			if procs[i] != nil {
 				procs[i].Stop()
+				procs[i].Close()
 			}
-			os.RemoveAll(tmpdirs[i])
 		}
 	}()
 	for i := range procs {
 		args := []string{
-			e2e.BinDir + "/etcd",
+			e2e.BinPath.Etcd,
 			"--name", fmt.Sprintf("e%d", i),
 			"--listen-client-urls", "http://0.0.0.0:0",
 			"--data-dir", tmpdirs[i],
@@ -82,16 +75,13 @@ func TestEtcdMultiPeer(t *testing.T) {
 			"--initial-cluster", ic,
 		}
 		p, err := e2e.SpawnCmd(args, nil)
-		if err != nil {
-			t.Fatal(err)
-		}
+		require.NoError(t, err)
 		procs[i] = p
 	}
 
 	for _, p := range procs {
-		if err := e2e.WaitReadyExpectProc(p, e2e.EtcdServerReadyLines); err != nil {
-			t.Fatal(err)
-		}
+		err := e2e.WaitReadyExpectProc(context.TODO(), p, e2e.EtcdServerReadyLines)
+		require.NoError(t, err)
 	}
 }
 
@@ -99,14 +89,10 @@ func TestEtcdMultiPeer(t *testing.T) {
 func TestEtcdUnixPeers(t *testing.T) {
 	e2e.SkipInShortMode(t)
 
-	d, err := ioutil.TempDir("", "e1.etcd")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(d)
+	d := t.TempDir()
 	proc, err := e2e.SpawnCmd(
 		[]string{
-			e2e.BinDir + "/etcd",
+			e2e.BinPath.Etcd,
 			"--data-dir", d,
 			"--name", "e1",
 			"--listen-peer-urls", "unix://etcd.unix:1",
@@ -115,15 +101,9 @@ func TestEtcdUnixPeers(t *testing.T) {
 		}, nil,
 	)
 	defer os.Remove("etcd.unix:1")
-	if err != nil {
-		t.Fatal(err)
-	}
-	if err = e2e.WaitReadyExpectProc(proc, e2e.EtcdServerReadyLines); err != nil {
-		t.Fatal(err)
-	}
-	if err = proc.Stop(); err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
+	require.NoError(t, e2e.WaitReadyExpectProc(context.TODO(), proc, e2e.EtcdServerReadyLines))
+	require.NoError(t, proc.Stop())
 }
 
 // TestEtcdListenMetricsURLsWithMissingClientTLSInfo checks that the HTTPs listen metrics URL
@@ -143,7 +123,7 @@ func TestEtcdListenMetricsURLsWithMissingClientTLSInfo(t *testing.T) {
 	listenMetricsURL := fmt.Sprintf("https://localhost:%d", e2e.EtcdProcessBasePort+2)
 
 	commonArgs := []string{
-		e2e.BinPath,
+		e2e.BinPath.Etcd,
 		"--name", "e0",
 		"--data-dir", tempDir,
 
@@ -164,18 +144,13 @@ func TestEtcdListenMetricsURLsWithMissingClientTLSInfo(t *testing.T) {
 	}
 
 	proc, err := e2e.SpawnCmd(commonArgs, nil)
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 	defer func() {
-		// Don't check the error returned by Stop(), as we expect the process to exit with an error.
-		_ = proc.Stop()
+		require.NoError(t, proc.Stop())
 		_ = proc.Close()
 	}()
 
-	if err := e2e.WaitReadyExpectProc(proc, []string{embed.ErrMissingClientTLSInfoForMetricsURL.Error()}); err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, e2e.WaitReadyExpectProc(context.TODO(), proc, []string{embed.ErrMissingClientTLSInfoForMetricsURL.Error()}))
 }
 
 // TestEtcdPeerCNAuth checks that the inter peer auth based on CN of cert is working correctly.
@@ -185,11 +160,7 @@ func TestEtcdPeerCNAuth(t *testing.T) {
 	peers, tmpdirs := make([]string, 3), make([]string, 3)
 	for i := range peers {
 		peers[i] = fmt.Sprintf("e%d=https://127.0.0.1:%d", i, e2e.EtcdProcessBasePort+i)
-		d, err := ioutil.TempDir("", fmt.Sprintf("e%d.etcd", i))
-		if err != nil {
-			t.Fatal(err)
-		}
-		tmpdirs[i] = d
+		tmpdirs[i] = t.TempDir()
 	}
 	ic := strings.Join(peers, ",")
 
@@ -198,15 +169,15 @@ func TestEtcdPeerCNAuth(t *testing.T) {
 		for i := range procs {
 			if procs[i] != nil {
 				procs[i].Stop()
+				procs[i].Close()
 			}
-			os.RemoveAll(tmpdirs[i])
 		}
 	}()
 
 	// node 0 and 1 have a cert with the correct CN, node 2 doesn't
 	for i := range procs {
 		commonArgs := []string{
-			e2e.BinDir + "/etcd",
+			e2e.BinPath.Etcd,
 			"--name", fmt.Sprintf("e%d", i),
 			"--listen-client-urls", "http://0.0.0.0:0",
 			"--data-dir", tmpdirs[i],
@@ -242,9 +213,7 @@ func TestEtcdPeerCNAuth(t *testing.T) {
 		commonArgs = append(commonArgs, args...)
 
 		p, err := e2e.SpawnCmd(commonArgs, nil)
-		if err != nil {
-			t.Fatal(err)
-		}
+		require.NoError(t, err)
 		procs[i] = p
 	}
 
@@ -255,9 +224,8 @@ func TestEtcdPeerCNAuth(t *testing.T) {
 		} else {
 			expect = []string{"remote error: tls: bad certificate"}
 		}
-		if err := e2e.WaitReadyExpectProc(p, expect); err != nil {
-			t.Fatal(err)
-		}
+		err := e2e.WaitReadyExpectProc(context.TODO(), p, expect)
+		require.NoError(t, err)
 	}
 }
 
@@ -286,7 +254,7 @@ func TestEtcdPeerMultiCNAuth(t *testing.T) {
 	// node 0 and 1 have a cert with one of the correct CNs, node 2 doesn't
 	for i := range procs {
 		commonArgs := []string{
-			e2e.BinDir + "/etcd",
+			e2e.BinPath.Etcd,
 			"--name", fmt.Sprintf("e%d", i),
 			"--listen-client-urls", "http://0.0.0.0:0",
 			"--data-dir", tmpdirs[i],
@@ -332,9 +300,7 @@ func TestEtcdPeerMultiCNAuth(t *testing.T) {
 
 		commonArgs = append(commonArgs, args...)
 		p, err := e2e.SpawnCmd(commonArgs, nil)
-		if err != nil {
-			t.Fatal(err)
-		}
+		require.NoError(t, err)
 		procs[i] = p
 	}
 
@@ -345,9 +311,8 @@ func TestEtcdPeerMultiCNAuth(t *testing.T) {
 		} else {
 			expect = []string{"remote error: tls: bad certificate"}
 		}
-		if err := e2e.WaitReadyExpectProc(p, expect); err != nil {
-			t.Fatal(err)
-		}
+		err := e2e.WaitReadyExpectProc(context.TODO(), p, expect)
+		require.NoError(t, err)
 	}
 }
 
@@ -358,11 +323,7 @@ func TestEtcdPeerNameAuth(t *testing.T) {
 	peers, tmpdirs := make([]string, 3), make([]string, 3)
 	for i := range peers {
 		peers[i] = fmt.Sprintf("e%d=https://127.0.0.1:%d", i, e2e.EtcdProcessBasePort+i)
-		d, err := ioutil.TempDir("", fmt.Sprintf("e%d.etcd", i))
-		if err != nil {
-			t.Fatal(err)
-		}
-		tmpdirs[i] = d
+		tmpdirs[i] = t.TempDir()
 	}
 	ic := strings.Join(peers, ",")
 
@@ -371,6 +332,7 @@ func TestEtcdPeerNameAuth(t *testing.T) {
 		for i := range procs {
 			if procs[i] != nil {
 				procs[i].Stop()
+				procs[i].Close()
 			}
 			os.RemoveAll(tmpdirs[i])
 		}
@@ -379,7 +341,7 @@ func TestEtcdPeerNameAuth(t *testing.T) {
 	// node 0 and 1 have a cert with the correct certificate name, node 2 doesn't
 	for i := range procs {
 		commonArgs := []string{
-			e2e.BinDir + "/etcd",
+			e2e.BinPath.Etcd,
 			"--name", fmt.Sprintf("e%d", i),
 			"--listen-client-urls", "http://0.0.0.0:0",
 			"--data-dir", tmpdirs[i],
@@ -411,9 +373,7 @@ func TestEtcdPeerNameAuth(t *testing.T) {
 		commonArgs = append(commonArgs, args...)
 
 		p, err := e2e.SpawnCmd(commonArgs, nil)
-		if err != nil {
-			t.Fatal(err)
-		}
+		require.NoError(t, err)
 		procs[i] = p
 	}
 
@@ -424,9 +384,114 @@ func TestEtcdPeerNameAuth(t *testing.T) {
 		} else {
 			expect = []string{"client certificate authentication failed"}
 		}
-		if err := e2e.WaitReadyExpectProc(p, expect); err != nil {
-			t.Fatal(err)
+		err := e2e.WaitReadyExpectProc(context.TODO(), p, expect)
+		require.NoError(t, err)
+	}
+}
+
+// TestEtcdPeerLocalAddr checks that the inter peer auth works with when
+// the member LocalAddr is set.
+func TestEtcdPeerLocalAddr(t *testing.T) {
+	e2e.SkipInShortMode(t)
+
+	nodeIP, err := getLocalIP()
+	t.Log("Using node IP", nodeIP)
+	require.NoError(t, err)
+
+	peers, tmpdirs := make([]string, 3), make([]string, 3)
+
+	for i := range peers {
+		peerIP := nodeIP
+		if i == 0 {
+			peerIP = "127.0.0.1"
 		}
+		peers[i] = fmt.Sprintf("e%d=https://%s:%d", i, peerIP, e2e.EtcdProcessBasePort+i)
+		tmpdirs[i] = t.TempDir()
+	}
+	procs := make([]*expect.ExpectProcess, len(peers))
+	defer func() {
+		for i := range procs {
+			if procs[i] != nil {
+				procs[i].Stop()
+				procs[i].Close()
+			}
+			os.RemoveAll(tmpdirs[i])
+		}
+	}()
+
+	tempDir := t.TempDir()
+	caFile, certFiles, keyFiles, err := generateCertsForIPs(tempDir, []net.IP{net.ParseIP("127.0.0.1"), net.ParseIP(nodeIP)})
+	require.NoError(t, err)
+
+	defer func() {
+		os.RemoveAll(tempDir)
+	}()
+
+	// node 0 (127.0.0.1) does not set `--feature-gates=SetMemberLocalAddr=true`,
+	// while nodes 1 and nodes 2 do.
+	//
+	// node 0's peer certificate is signed for 127.0.0.1, but it uses the host
+	// IP (by default) to communicate with peers, so they don't match.
+	// Accordingly, other peers will reject connections from node 0.
+	//
+	// Both node 1 and node 2's peer certificates are signed for the host IP,
+	// and they also communicate with peers using the host IP (explicitly set
+	// with --initial-advertise-peer-urls and
+	// --feature-gates=SetMemberLocalAddr=true), so node 0 has no issue connecting
+	// to them.
+	//
+	// Refer to https://github.com/etcd-io/etcd/issues/17068.
+	for i := range procs {
+		peerIP := nodeIP
+		if i == 0 {
+			peerIP = "127.0.0.1"
+		}
+		ic := strings.Join(peers, ",")
+		commonArgs := []string{
+			e2e.BinPath.Etcd,
+			"--name", fmt.Sprintf("e%d", i),
+			"--listen-client-urls", "http://0.0.0.0:0",
+			"--data-dir", tmpdirs[i],
+			"--advertise-client-urls", "http://0.0.0.0:0",
+			"--initial-advertise-peer-urls", fmt.Sprintf("https://%s:%d", peerIP, e2e.EtcdProcessBasePort+i),
+			"--listen-peer-urls", fmt.Sprintf("https://%s:%d,https://%s:%d", peerIP, e2e.EtcdProcessBasePort+i, peerIP, e2e.EtcdProcessBasePort+len(peers)+i),
+			"--initial-cluster", ic,
+		}
+
+		var args []string
+		if i == 0 {
+			args = []string{
+				"--peer-cert-file", certFiles[0],
+				"--peer-key-file", keyFiles[0],
+				"--peer-trusted-ca-file", caFile,
+				"--peer-client-cert-auth",
+			}
+		} else {
+			args = []string{
+				"--peer-cert-file", certFiles[1],
+				"--peer-key-file", keyFiles[1],
+				"--peer-trusted-ca-file", caFile,
+				"--peer-client-cert-auth",
+				"--feature-gates=SetMemberLocalAddr=true",
+			}
+		}
+
+		commonArgs = append(commonArgs, args...)
+
+		p, err := e2e.SpawnCmd(commonArgs, nil)
+		require.NoError(t, err)
+		procs[i] = p
+	}
+
+	for i, p := range procs {
+		var expect []string
+		if i == 0 {
+			expect = e2e.EtcdServerReadyLines
+		} else {
+			expect = []string{"x509: certificate is valid for 127.0.0.1, not "}
+		}
+		err := e2e.WaitReadyExpectProc(context.TODO(), p, expect)
+		require.NoError(t, err)
 	}
 }
 
@@ -434,7 +499,7 @@ func TestGrpcproxyAndCommonName(t *testing.T) {
 	e2e.SkipInShortMode(t)
 
 	argsWithNonEmptyCN := []string{
-		e2e.BinDir + "/etcd",
+		e2e.BinPath.Etcd,
 		"grpc-proxy",
 		"start",
 		"--cert", e2e.CertPath2,
@@ -443,7 +508,7 @@ func TestGrpcproxyAndCommonName(t *testing.T) {
 	}
 
 	argsWithEmptyCN := []string{
-		e2e.BinDir + "/etcd",
+		e2e.BinPath.Etcd,
 		"grpc-proxy",
 		"start",
 		"--cert", e2e.CertPath3,
@@ -451,10 +516,8 @@ func TestGrpcproxyAndCommonName(t *testing.T) {
 		"--cacert", e2e.CaPath,
 	}
 
-	err := e2e.SpawnWithExpect(argsWithNonEmptyCN, "cert has non empty Common Name")
-	if err != nil {
-		t.Errorf("Unexpected error: %s", err)
-	}
+	err := e2e.SpawnWithExpect(argsWithNonEmptyCN, expect.ExpectedResponse{Value: "cert has non empty Common Name"})
+	require.ErrorContains(t, err, "cert has non empty Common Name")
 
 	p, err := e2e.SpawnCmd(argsWithEmptyCN, nil)
 	defer func() {
@@ -463,9 +526,7 @@ func TestGrpcproxyAndCommonName(t *testing.T) {
 		}
 	}()
 
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 }
 
 func TestGrpcproxyAndListenCipherSuite(t *testing.T) {
@@ -478,7 +539,7 @@ func TestGrpcproxyAndListenCipherSuite(t *testing.T) {
 		{
 			name: "ArgsWithCipherSuites",
 			args: []string{
-				e2e.BinDir + "/etcd",
+				e2e.BinPath.Etcd,
 				"grpc-proxy",
 				"start",
 				"--listen-cipher-suites", "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256",
@@ -487,7 +548,7 @@ func TestGrpcproxyAndListenCipherSuite(t *testing.T) {
 		{
 			name: "ArgsWithoutCipherSuites",
 			args: []string{
-				e2e.BinDir + "/etcd",
+				e2e.BinPath.Etcd,
 				"grpc-proxy",
 				"start",
 				"--listen-cipher-suites", "",
@@ -498,12 +559,8 @@ func TestGrpcproxyAndListenCipherSuite(t *testing.T) {
 	for _, test := range cases {
 		t.Run(test.name, func(t *testing.T) {
 			pw, err := e2e.SpawnCmd(test.args, nil)
-			if err != nil {
-				t.Fatal(err)
-			}
-			if err = pw.Stop(); err != nil {
-				t.Fatal(err)
-			}
+			require.NoError(t, err)
+			require.NoError(t, pw.Stop())
 		})
 	}
 }
@@ -511,16 +568,67 @@ func TestGrpcproxyAndListenCipherSuite(t *testing.T) {
 func TestBootstrapDefragFlag(t *testing.T) {
 	e2e.SkipInShortMode(t)
 
-	proc, err := e2e.SpawnCmd([]string{e2e.BinDir + "/etcd", "--experimental-bootstrap-defrag-threshold-megabytes", "1000"}, nil)
-	if err != nil {
-		t.Fatal(err)
+	proc, err := e2e.SpawnCmd([]string{e2e.BinPath.Etcd, "--experimental-bootstrap-defrag-threshold-megabytes", "1000"}, nil)
+	require.NoError(t, err)
+	require.NoError(t, e2e.WaitReadyExpectProc(context.TODO(), proc, []string{"Skipping defragmentation"}))
+	require.NoError(t, proc.Stop())
+
+	// wait for the process to exit, otherwise test will have leaked goroutine
+	if err := proc.Close(); err != nil {
+		t.Logf("etcd process closed with error %v", err)
 	}
-	if err = e2e.WaitReadyExpectProc(proc, []string{"Skipping defragmentation"}); err != nil {
-		t.Fatal(err)
+}
+
+func TestSnapshotCatchupEntriesFlag(t *testing.T) {
+	e2e.SkipInShortMode(t)
+
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
+
+	proc, err := e2e.SpawnCmd([]string{e2e.BinPath.Etcd, "--experimental-snapshot-catchup-entries", "1000"}, nil)
+	require.NoError(t, err)
+	require.NoError(t, e2e.WaitReadyExpectProc(ctx, proc, []string{"\"snapshot-catchup-entries\":1000"}))
+	require.NoError(t, e2e.WaitReadyExpectProc(ctx, proc, []string{"serving client traffic"}))
+	require.NoError(t, proc.Stop())
+
+	// wait for the process to exit, otherwise test will have leaked goroutine
+	if err := proc.Close(); err != nil {
+		t.Logf("etcd process closed with error %v", err)
 	}
-	if err = proc.Stop(); err != nil {
-		t.Fatal(err)
+}
+
+// TestEtcdHealthyWithTinySnapshotCatchupEntries ensures multi-node etcd cluster remains healthy with 1 snapshot catch up entry
+func TestEtcdHealthyWithTinySnapshotCatchupEntries(t *testing.T) {
+	e2e.BeforeTest(t)
+	epc, err := e2e.NewEtcdProcessCluster(context.TODO(), t,
+		e2e.WithClusterSize(3),
+		e2e.WithSnapshotCount(1),
+		e2e.WithSnapshotCatchUpEntries(1),
+	)
+	require.NoErrorf(t, err, "could not start etcd process cluster (%v)", err)
+	t.Cleanup(func() {
+		if errC := epc.Close(); errC != nil {
+			t.Fatalf("error closing etcd processes (%v)", errC)
+		}
+	})
+
+	// simulate 10 clients keep writing to etcd in parallel with no error
+	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
+	defer cancel()
+	g, ctx := errgroup.WithContext(ctx)
+	for i := 0; i < 10; i++ {
+		clientID := i
+		g.Go(func() error {
+			cc := epc.Etcdctl()
+			for j := 0; j < 100; j++ {
+				if err := cc.Put(ctx, "foo", fmt.Sprintf("bar%d", clientID), config.PutOptions{}); err != nil {
+					return err
+				}
+			}
+			return nil
+		})
 	}
+	require.NoError(t, g.Wait())
 }
 
 func TestEtcdTLSVersion(t *testing.T) {
@@ -529,7 +637,7 @@ func TestEtcdTLSVersion(t *testing.T) {
 	d := t.TempDir()
 	proc, err := e2e.SpawnCmd(
 		[]string{
-			e2e.BinDir + "/etcd",
+			e2e.BinPath.Etcd,
 			"--data-dir", d,
 			"--name", "e1",
 			"--listen-client-urls", "https://0.0.0.0:0",
@@ -547,7 +655,83 @@ func TestEtcdTLSVersion(t *testing.T) {
 		}, nil,
 	)
 	assert.NoError(t, err)
-	assert.NoError(t, e2e.WaitReadyExpectProc(proc, e2e.EtcdServerReadyLines), "did not receive expected output from etcd process")
+	assert.NoErrorf(t, e2e.WaitReadyExpectProc(context.TODO(), proc, e2e.EtcdServerReadyLines), "did not receive expected output from etcd process")
 	assert.NoError(t, proc.Stop())
 
+	proc.Wait() // ensure the port has been released
+	proc.Close()
+}
+
+// TestEtcdDeprecatedFlags checks that etcd will print warning messages if deprecated flags are set.
+func TestEtcdDeprecatedFlags(t *testing.T) {
+	e2e.SkipInShortMode(t)
+
+	commonArgs := []string{
+		e2e.BinPath.Etcd,
+		"--name", "e1",
+	}
+
+	testCases := []struct {
+		name        string
+		args        []string
+		expectedMsg string
+	}{
+		{
+			name:        "snapshot-count",
+			args:        append(commonArgs, "--snapshot-count=100"),
+			expectedMsg: "--snapshot-count is deprecated in 3.6 and will be decommissioned in 3.7",
+		},
+		{
+			name:        "max-snapshots",
+			args:        append(commonArgs, "--max-snapshots=10"),
+			expectedMsg: "--max-snapshots is deprecated in 3.6 and will be decommissioned in 3.7",
+		},
+		{
+			name:        "v2-deprecation",
+			args:        append(commonArgs, "--v2-deprecation", "write-only-drop-data"),
+			expectedMsg: "--v2-deprecation is deprecated and scheduled for removal in v3.8. The default value is enforced, ignoring user input",
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			proc, err := e2e.SpawnCmd(
+				tc.args, nil,
+			)
+			require.NoError(t, err)
+			require.NoError(t, e2e.WaitReadyExpectProc(context.TODO(), proc, []string{tc.expectedMsg}))
+			require.NoError(t, proc.Stop())
+
+			proc.Wait() // ensure the port has been released
+			proc.Close()
+		})
+	}
+}
+
+// TestV2DeprecationEnforceDefaultValue verifies that etcd enforces the default V2Deprecation level
+// and ignores users input.
+func TestV2DeprecationEnforceDefaultValue(t *testing.T) {
+	e2e.SkipInShortMode(t)
+
+	commonArgs := []string{
+		e2e.BinPath.Etcd,
+		"--name", "e1",
+	}
+
+	validV2DeprecationLevels := []string{"write-only", "write-only-drop-data", "gone"}
+	expectedDeprecationLevelMsg := `"v2-deprecation":"write-only"`
+
+	for _, optionLevel := range validV2DeprecationLevels {
+		t.Run(optionLevel, func(t *testing.T) {
+			proc, err := e2e.SpawnCmd(
+				append(commonArgs, "--v2-deprecation", optionLevel), nil,
+			)
+			require.NoError(t, err)
+			require.NoError(t, e2e.WaitReadyExpectProc(context.TODO(), proc, []string{expectedDeprecationLevelMsg}))
+			require.NoError(t, proc.Stop())
+
+			proc.Wait() // ensure the port has been released
+			proc.Close()
+		})
+	}
 }
