@@ -1,4 +1,4 @@
-// Copyright 2024 The etcd Authors
+// Copyright 2023 The etcd Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
 	"go.etcd.io/etcd/api/v3/v3rpc/rpctypes"
@@ -59,16 +60,14 @@ func testLeaseRevokeIssue(t *testing.T, clusterSize int, connectToOneFollower bo
 	ctx := context.Background()
 
 	t.Log("Starting a new etcd cluster")
-	epc, err := e2e.NewEtcdProcessCluster(t, &e2e.EtcdProcessClusterConfig{
-		ClusterSize:         clusterSize,
-		GoFailEnabled:       true,
-		GoFailClientTimeout: 40 * time.Second,
-	})
+	epc, err := e2e.NewEtcdProcessCluster(ctx, t,
+		e2e.WithClusterSize(clusterSize),
+		e2e.WithGoFailEnabled(true),
+		e2e.WithGoFailClientTimeout(40*time.Second),
+	)
 	require.NoError(t, err)
 	defer func() {
-		if errC := epc.Close(); errC != nil {
-			t.Fatalf("error closing etcd processes (%v)", errC)
-		}
+		require.NoErrorf(t, epc.Close(), "error closing etcd processes")
 	}()
 
 	leaderIdx := epc.WaitLeader(t)
@@ -93,7 +92,7 @@ func testLeaseRevokeIssue(t *testing.T, clusterSize int, connectToOneFollower bo
 
 	resp, err := client.Status(ctx, epsForNormalOperations[0])
 	require.NoError(t, err)
-	oldLeaderId := resp.Leader
+	oldLeaderID := resp.Leader
 
 	t.Log("Creating a new lease")
 	leaseRsp, err := client.Grant(ctx, 20)
@@ -107,7 +106,7 @@ func testLeaseRevokeIssue(t *testing.T, clusterSize int, connectToOneFollower bo
 		defer close(doneC)
 
 		respC, kerr := clientForKeepAlive.KeepAlive(ctx, leaseRsp.ID)
-		require.NoError(t, kerr)
+		assert.NoError(t, kerr)
 		// ensure we have received the first response from the server
 		<-respC
 		startC <- struct{}{}
@@ -129,11 +128,11 @@ func testLeaseRevokeIssue(t *testing.T, clusterSize int, connectToOneFollower bo
 	require.NoError(t, err)
 
 	cctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	t.Logf("Waiting for a new leader to be elected, old leader index: %d, old leader ID: %d", leaderIdx, oldLeaderId)
+	t.Logf("Waiting for a new leader to be elected, old leader index: %d, old leader ID: %d", leaderIdx, oldLeaderID)
 	testutils.ExecuteUntil(cctx, t, func() {
 		for {
 			resp, err = client.Status(ctx, epsForNormalOperations[0])
-			if err == nil && resp.Leader != oldLeaderId {
+			if err == nil && resp.Leader != oldLeaderID {
 				t.Logf("A new leader has already been elected, new leader index: %d", resp.Leader)
 				return
 			}
@@ -160,7 +159,7 @@ func testLeaseRevokeIssue(t *testing.T, clusterSize int, connectToOneFollower bo
 	t.Log("Confirming the lease isn't revoked")
 	leases, err := client.Leases(ctx)
 	require.NoError(t, err)
-	require.Equal(t, 1, len(leases.Leases))
+	require.Len(t, leases.Leases, 1)
 
 	t.Log("Waiting for the keepAlive goroutine to exit")
 	close(stopC)
@@ -173,12 +172,11 @@ func TestLeaseRevokeDuringRenew(t *testing.T) {
 	ctx := t.Context()
 
 	t.Log("Starting a new etcd cluster")
-
-	epc, err := e2e.NewEtcdProcessCluster(t, &e2e.EtcdProcessClusterConfig{
-		ClusterSize:         1,
-		GoFailEnabled:       true,
-		GoFailClientTimeout: 40 * time.Second,
-	})
+	epc, err := e2e.NewEtcdProcessCluster(ctx, t,
+		e2e.WithClusterSize(1),
+		e2e.WithGoFailEnabled(true),
+		e2e.WithGoFailClientTimeout(40*time.Second),
+	)
 	require.NoError(t, err)
 	defer func() {
 		require.NoErrorf(t, epc.Close(), "error closing etcd processes")
