@@ -1,4 +1,4 @@
-// Copyright 2024 The etcd Authors
+// Copyright 2022 The etcd Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -23,6 +23,13 @@ const (
 	LastVersion         ClusterVersion = "last-version"
 )
 
+func (cv ClusterVersion) String() string {
+	if cv == CurrentVersion {
+		return "current-version"
+	}
+	return string(cv)
+}
+
 type ClusterContext struct {
 	Version ClusterVersion
 }
