@@ -16,6 +16,7 @@ package clientv3test
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"math/rand"
 	"reflect"
@@ -24,20 +25,22 @@ import (
 	"testing"
 	"time"
 
+	"github.com/google/go-cmp/cmp"
 	"github.com/stretchr/testify/require"
+	"google.golang.org/grpc/metadata"
+
 	mvccpb "go.etcd.io/etcd/api/v3/mvccpb"
 	"go.etcd.io/etcd/api/v3/v3rpc/rpctypes"
 	"go.etcd.io/etcd/api/v3/version"
 	clientv3 "go.etcd.io/etcd/client/v3"
 	"go.etcd.io/etcd/server/v3/etcdserver/api/v3rpc"
-	"go.etcd.io/etcd/tests/v3/integration"
-	"google.golang.org/grpc/metadata"
+	integration2 "go.etcd.io/etcd/tests/v3/framework/integration"
 )
 
 type watcherTest func(*testing.T, *watchctx)
 
 type watchctx struct {
-	clus          *integration.ClusterV3
+	clus          *integration2.Cluster
 	w             clientv3.Watcher
 	kv            clientv3.KV
 	wclientMember int
@@ -46,9 +49,9 @@ type watchctx struct {
 }
 
 func runWatchTest(t *testing.T, f watcherTest) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 3, UseBridge: true})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 3, UseBridge: true})
 	defer clus.Terminate(t)
 
 	wclientMember := rand.Intn(3)
@@ -75,6 +78,12 @@ func testWatchMultiWatcher(t *testing.T, wctx *watchctx) {
 	keys := []string{"foo", "bar", "baz"}
 
 	donec := make(chan struct{})
+	// wait for watcher shutdown
+	defer func() {
+		for i := 0; i < len(keys)+1; i++ {
+			<-donec
+		}
+	}()
 	readyc := make(chan struct{})
 	for _, k := range keys {
 		// key watcher
@@ -105,7 +114,7 @@ func testWatchMultiWatcher(t *testing.T, wctx *watchctx) {
 			t.Errorf("expected watcher channel, got nil")
 		}
 		readyc <- struct{}{}
-		evs := []*clientv3.Event{}
+		var evs []*clientv3.Event
 		for i := 0; i < numKeyUpdates*2; i++ {
 			resp, ok := <-prefixc
 			if !ok {
@@ -115,14 +124,14 @@ func testWatchMultiWatcher(t *testing.T, wctx *watchctx) {
 		}
 
 		// check response
-		expected := []string{}
+		var expected []string
 		bkeys := []string{"bar", "baz"}
 		for _, k := range bkeys {
 			for i := 0; i < numKeyUpdates; i++ {
 				expected = append(expected, fmt.Sprintf("%s-%d", k, i))
 			}
 		}
-		got := []string{}
+		var got []string
 		for _, ev := range evs {
 			got = append(got, string(ev.Kv.Value))
 		}
@@ -152,15 +161,10 @@ func testWatchMultiWatcher(t *testing.T, wctx *watchctx) {
 	for i := 0; i < numKeyUpdates; i++ {
 		for _, k := range keys {
 			v := fmt.Sprintf("%s-%d", k, i)
-			if _, err := wctx.kv.Put(ctx, k, v); err != nil {
-				t.Fatal(err)
-			}
+			_, err := wctx.kv.Put(ctx, k, v)
+			require.NoError(t, err)
 		}
 	}
-	// wait for watcher shutdown
-	for i := 0; i < len(keys)+1; i++ {
-		<-donec
-	}
 }
 
 // TestWatchRange tests watcher creates ranges
@@ -212,9 +216,8 @@ func testWatchReconnRequest(t *testing.T, wctx *watchctx) {
 	// spinning on dropping connections may trigger a leader election
 	// due to resource starvation; l-read to ensure the cluster is stable
 	ctx, cancel := context.WithTimeout(context.TODO(), 30*time.Second)
-	if _, err := wctx.kv.Get(ctx, "_"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := wctx.kv.Get(ctx, "_")
+	require.NoError(t, err)
 	cancel()
 
 	// ensure watcher works
@@ -300,15 +303,12 @@ func TestWatchCancelRunning(t *testing.T) {
 }
 
 func testWatchCancelRunning(t *testing.T, wctx *watchctx) {
-	integration.BeforeTest(t)
-
 	ctx, cancel := context.WithCancel(context.Background())
 	if wctx.ch = wctx.w.Watch(ctx, "a"); wctx.ch == nil {
 		t.Fatalf("expected non-nil watcher channel")
 	}
-	if _, err := wctx.kv.Put(ctx, "a", "a"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := wctx.kv.Put(ctx, "a", "a")
+	require.NoError(t, err)
 	cancel()
 	select {
 	case <-time.After(time.Second):
@@ -331,9 +331,8 @@ func testWatchCancelRunning(t *testing.T, wctx *watchctx) {
 }
 
 func putAndWatch(t *testing.T, wctx *watchctx, key, val string) {
-	if _, err := wctx.kv.Put(context.TODO(), key, val); err != nil {
-		t.Fatal(err)
-	}
+	_, err := wctx.kv.Put(context.TODO(), key, val)
+	require.NoError(t, err)
 	select {
 	case <-time.After(5 * time.Second):
 		t.Fatalf("watch timed out")
@@ -350,24 +349,26 @@ func putAndWatch(t *testing.T, wctx *watchctx, key, val string) {
 	}
 }
 
-func TestWatchResumeInitRev(t *testing.T) {
-	integration.BeforeTest(t)
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1, UseBridge: true})
+// TestWatchResumeAfterDisconnect tests watch resume after member disconnects then connects.
+// It ensures that correct events are returned corresponding to the start revision.
+func TestWatchResumeAfterDisconnect(t *testing.T) {
+	integration2.BeforeTest(t)
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1, UseBridge: true})
 	defer clus.Terminate(t)
 
 	cli := clus.Client(0)
-	if _, err := cli.Put(context.TODO(), "b", "2"); err != nil {
-		t.Fatal(err)
-	}
-	if _, err := cli.Put(context.TODO(), "a", "3"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := cli.Put(context.TODO(), "b", "2")
+	require.NoError(t, err)
+	_, err = cli.Put(context.TODO(), "a", "3")
+	require.NoError(t, err)
 	// if resume is broken, it'll pick up this key first instead of a=3
-	if _, err := cli.Put(context.TODO(), "a", "4"); err != nil {
-		t.Fatal(err)
-	}
+	_, err = cli.Put(context.TODO(), "a", "4")
+	require.NoError(t, err)
 
+	// watch from revision 1
 	wch := clus.Client(0).Watch(context.Background(), "a", clientv3.WithRev(1), clientv3.WithCreatedNotify())
+	// response for the create watch request, no events are in this response
+	// the current revision of etcd should be 4
 	if resp, ok := <-wch; !ok || resp.Header.Revision != 4 {
 		t.Fatalf("got (%v, %v), expected create notification rev=4", resp, ok)
 	}
@@ -389,12 +390,16 @@ func TestWatchResumeInitRev(t *testing.T) {
 		if !ok {
 			t.Fatal("unexpected watch close")
 		}
-		if len(resp.Events) == 0 {
-			t.Fatal("expected event on watch")
+		// Events should be put(a, 3) and put(a, 4)
+		if len(resp.Events) != 2 {
+			t.Fatal("expected two events on watch")
 		}
 		if string(resp.Events[0].Kv.Value) != "3" {
 			t.Fatalf("expected value=3, got event %+v", resp.Events[0])
 		}
+		if string(resp.Events[1].Kv.Value) != "4" {
+			t.Fatalf("expected value=4, got event %+v", resp.Events[1])
+		}
 	case <-time.After(5 * time.Second):
 		t.Fatal("watch timed out")
 	}
@@ -406,9 +411,9 @@ func TestWatchResumeInitRev(t *testing.T) {
 // either a compaction error or all keys by staying in sync before the compaction
 // is finally applied.
 func TestWatchResumeCompacted(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 3, UseBridge: true})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 3, UseBridge: true})
 	defer clus.Terminate(t)
 
 	// create a waiting watcher at rev 1
@@ -421,27 +426,17 @@ func TestWatchResumeCompacted(t *testing.T) {
 	}
 	clus.Members[0].Stop(t)
 
-	ticker := time.After(time.Second * 10)
-	for clus.WaitLeader(t) <= 0 {
-		select {
-		case <-ticker:
-			t.Fatalf("failed to wait for new leader")
-		default:
-			time.Sleep(10 * time.Millisecond)
-		}
-	}
+	clus.WaitLeader(t)
 
 	// put some data and compact away
 	numPuts := 5
 	kv := clus.Client(1)
 	for i := 0; i < numPuts; i++ {
-		if _, err := kv.Put(context.TODO(), "foo", "bar"); err != nil {
-			t.Fatal(err)
-		}
-	}
-	if _, err := kv.Compact(context.TODO(), 3); err != nil {
-		t.Fatal(err)
+		_, err := kv.Put(context.TODO(), "foo", "bar")
+		require.NoError(t, err)
 	}
+	_, err := kv.Compact(context.TODO(), 3)
+	require.NoError(t, err)
 
 	clus.Members[0].Restart(t)
 
@@ -470,7 +465,7 @@ func TestWatchResumeCompacted(t *testing.T) {
 		if wresp.Err() == nil {
 			continue
 		}
-		if wresp.Err() != rpctypes.ErrCompacted {
+		if !errors.Is(wresp.Err(), rpctypes.ErrCompacted) {
 			t.Fatalf("wresp.Err() expected %v, got %+v", rpctypes.ErrCompacted, wresp.Err())
 		}
 		break
@@ -493,24 +488,22 @@ func TestWatchResumeCompacted(t *testing.T) {
 // TestWatchCompactRevision ensures the CompactRevision error is given on a
 // compaction event ahead of a watcher.
 func TestWatchCompactRevision(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer clus.Terminate(t)
 
 	// set some keys
 	kv := clus.RandClient()
 	for i := 0; i < 5; i++ {
-		if _, err := kv.Put(context.TODO(), "foo", "bar"); err != nil {
-			t.Fatal(err)
-		}
+		_, err := kv.Put(context.TODO(), "foo", "bar")
+		require.NoError(t, err)
 	}
 
 	w := clus.RandClient()
 
-	if _, err := kv.Compact(context.TODO(), 4); err != nil {
-		t.Fatal(err)
-	}
+	_, err := kv.Compact(context.TODO(), 4)
+	require.NoError(t, err)
 	wch := w.Watch(context.Background(), "foo", clientv3.WithRev(2))
 
 	// get compacted error message
@@ -518,7 +511,7 @@ func TestWatchCompactRevision(t *testing.T) {
 	if !ok {
 		t.Fatalf("expected wresp, but got closed channel")
 	}
-	if wresp.Err() != rpctypes.ErrCompacted {
+	if !errors.Is(wresp.Err(), rpctypes.ErrCompacted) {
 		t.Fatalf("wresp.Err() expected %v, but got %v", rpctypes.ErrCompacted, wresp.Err())
 	}
 	if !wresp.Canceled {
@@ -535,7 +528,7 @@ func TestWatchWithProgressNotify(t *testing.T)        { testWatchWithProgressNot
 func TestWatchWithProgressNotifyNoEvent(t *testing.T) { testWatchWithProgressNotify(t, false) }
 
 func testWatchWithProgressNotify(t *testing.T, watchOnPut bool) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
 	// accelerate report interval so test terminates quickly
 	oldpi := v3rpc.GetProgressReportInterval()
@@ -544,7 +537,7 @@ func testWatchWithProgressNotify(t *testing.T, watchOnPut bool) {
 	pi := 3 * time.Second
 	defer func() { v3rpc.SetProgressReportInterval(oldpi) }()
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 3})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 3})
 	defer clus.Terminate(t)
 
 	wc := clus.RandClient()
@@ -565,9 +558,8 @@ func testWatchWithProgressNotify(t *testing.T, watchOnPut bool) {
 	}
 
 	kvc := clus.RandClient()
-	if _, err := kvc.Put(context.TODO(), "foox", "bar"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := kvc.Put(context.TODO(), "foox", "bar")
+	require.NoError(t, err)
 
 	select {
 	case resp := <-rch:
@@ -575,8 +567,10 @@ func testWatchWithProgressNotify(t *testing.T, watchOnPut bool) {
 			t.Fatalf("resp.Header.Revision expected 2, got %d", resp.Header.Revision)
 		}
 		if watchOnPut { // wait for put if watch on the put key
-			ev := []*clientv3.Event{{Type: clientv3.EventTypePut,
-				Kv: &mvccpb.KeyValue{Key: []byte("foox"), Value: []byte("bar"), CreateRevision: 2, ModRevision: 2, Version: 1}}}
+			ev := []*clientv3.Event{{
+				Type: clientv3.EventTypePut,
+				Kv:   &mvccpb.KeyValue{Key: []byte("foox"), Value: []byte("bar"), CreateRevision: 2, ModRevision: 2, Version: 1},
+			}}
 			if !reflect.DeepEqual(ev, resp.Events) {
 				t.Fatalf("expected %+v, got %+v", ev, resp.Events)
 			}
@@ -589,11 +583,11 @@ func testWatchWithProgressNotify(t *testing.T, watchOnPut bool) {
 }
 
 func TestConfigurableWatchProgressNotifyInterval(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
 	progressInterval := 200 * time.Millisecond
-	clus := integration.NewClusterV3(t,
-		&integration.ClusterConfig{
+	clus := integration2.NewCluster(t,
+		&integration2.ClusterConfig{
 			Size:                        3,
 			WatchProgressNotifyInterval: progressInterval,
 		})
@@ -615,7 +609,7 @@ func TestConfigurableWatchProgressNotifyInterval(t *testing.T) {
 }
 
 func TestWatchRequestProgress(t *testing.T) {
-	if integration.ThroughProxy {
+	if integration2.ThroughProxy {
 		t.Skipf("grpc-proxy does not support WatchProgress yet")
 	}
 	testCases := []struct {
@@ -629,11 +623,11 @@ func TestWatchRequestProgress(t *testing.T) {
 
 	for _, c := range testCases {
 		t.Run(c.name, func(t *testing.T) {
-			integration.BeforeTest(t)
+			integration2.BeforeTest(t)
 
 			watchTimeout := 3 * time.Second
 
-			clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 3})
+			clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 3})
 			defer clus.Terminate(t)
 
 			wc := clus.RandClient()
@@ -645,9 +639,7 @@ func TestWatchRequestProgress(t *testing.T) {
 			}
 
 			_, err := wc.Put(context.Background(), "/a", "1")
-			if err != nil {
-				t.Fatal(err)
-			}
+			require.NoError(t, err)
 
 			for _, rch := range watchChans {
 				select {
@@ -662,14 +654,9 @@ func TestWatchRequestProgress(t *testing.T) {
 
 			// put a value not being watched to increment revision
 			_, err = wc.Put(context.Background(), "x", "1")
-			if err != nil {
-				t.Fatal(err)
-			}
+			require.NoError(t, err)
 
-			err = wc.RequestProgress(context.Background())
-			if err != nil {
-				t.Fatal(err)
-			}
+			require.NoError(t, wc.RequestProgress(context.Background()))
 
 			// verify all watch channels receive a progress notify
 			for _, rch := range watchChans {
@@ -690,9 +677,9 @@ func TestWatchRequestProgress(t *testing.T) {
 }
 
 func TestWatchEventType(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	cluster := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	cluster := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer cluster.Terminate(t)
 
 	client := cluster.RandClient()
@@ -764,9 +751,9 @@ func TestWatchEventType(t *testing.T) {
 }
 
 func TestWatchErrConnClosed(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer clus.Terminate(t)
 
 	cli := clus.Client(0)
@@ -781,40 +768,36 @@ func TestWatchErrConnClosed(t *testing.T) {
 		}
 	}()
 
-	if err := cli.ActiveConnection().Close(); err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, cli.ActiveConnection().Close())
 	clus.TakeClient(0)
 
 	select {
-	case <-time.After(integration.RequestWaitTimeout):
+	case <-time.After(integration2.RequestWaitTimeout):
 		t.Fatal("wc.Watch took too long")
 	case <-donec:
 	}
 }
 
 func TestWatchAfterClose(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer clus.Terminate(t)
 
 	cli := clus.Client(0)
 	clus.TakeClient(0)
-	if err := cli.Close(); err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, cli.Close())
 
 	donec := make(chan struct{})
 	go func() {
 		cli.Watch(context.TODO(), "foo")
-		if err := cli.Close(); err != nil && err != context.Canceled {
+		if err := cli.Close(); err != nil && !errors.Is(err, context.Canceled) {
 			t.Errorf("expected %v, got %v", context.Canceled, err)
 		}
 		close(donec)
 	}()
 	select {
-	case <-time.After(integration.RequestWaitTimeout):
+	case <-time.After(integration2.RequestWaitTimeout):
 		t.Fatal("wc.Watch took too long")
 	case <-donec:
 	}
@@ -822,9 +805,9 @@ func TestWatchAfterClose(t *testing.T) {
 
 // TestWatchWithRequireLeader checks the watch channel closes when no leader.
 func TestWatchWithRequireLeader(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 3})
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 3})
 	defer clus.Terminate(t)
 
 	// Put a key for the non-require leader watch to read as an event.
@@ -832,9 +815,8 @@ func TestWatchWithRequireLeader(t *testing.T) {
 	// ensure that it receives the update so watching after killing quorum
 	// is guaranteed to have the key.
 	liveClient := clus.Client(0)
-	if _, err := liveClient.Put(context.TODO(), "foo", "bar"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := liveClient.Put(context.TODO(), "foo", "bar")
+	require.NoError(t, err)
 
 	clus.Members[1].Stop(t)
 	clus.Members[2].Stop(t)
@@ -857,10 +839,10 @@ func TestWatchWithRequireLeader(t *testing.T) {
 		if !ok {
 			t.Fatalf("expected %v watch channel, got closed channel", rpctypes.ErrNoLeader)
 		}
-		if resp.Err() != rpctypes.ErrNoLeader {
+		if !errors.Is(resp.Err(), rpctypes.ErrNoLeader) {
 			t.Fatalf("expected %v watch response error, got %+v", rpctypes.ErrNoLeader, resp)
 		}
-	case <-time.After(integration.RequestWaitTimeout):
+	case <-time.After(integration2.RequestWaitTimeout):
 		t.Fatal("watch without leader took too long to close")
 	}
 
@@ -869,7 +851,7 @@ func TestWatchWithRequireLeader(t *testing.T) {
 		if ok {
 			t.Fatalf("expected closed channel, got response %v", resp)
 		}
-	case <-time.After(integration.RequestWaitTimeout):
+	case <-time.After(integration2.RequestWaitTimeout):
 		t.Fatal("waited too long for channel to close")
 	}
 
@@ -882,13 +864,9 @@ func TestWatchWithRequireLeader(t *testing.T) {
 		`type="stream"`,
 		fmt.Sprintf(`client_api_version="%v"`, version.APIVersion),
 	)
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 	cv, err := strconv.ParseInt(cnt, 10, 32)
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 	if cv < 2 { // >2 when retried
 		t.Fatalf("expected at least 2, got %q", cnt)
 	}
@@ -896,9 +874,9 @@ func TestWatchWithRequireLeader(t *testing.T) {
 
 // TestWatchWithFilter checks that watch filtering works.
 func TestWatchWithFilter(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	cluster := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	cluster := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer cluster.Terminate(t)
 
 	client := cluster.RandClient()
@@ -907,12 +885,10 @@ func TestWatchWithFilter(t *testing.T) {
 	wcNoPut := client.Watch(ctx, "a", clientv3.WithFilterPut())
 	wcNoDel := client.Watch(ctx, "a", clientv3.WithFilterDelete())
 
-	if _, err := client.Put(ctx, "a", "abc"); err != nil {
-		t.Fatal(err)
-	}
-	if _, err := client.Delete(ctx, "a"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := client.Put(ctx, "a", "abc")
+	require.NoError(t, err)
+	_, err = client.Delete(ctx, "a")
+	require.NoError(t, err)
 
 	npResp := <-wcNoPut
 	if len(npResp.Events) != 1 || npResp.Events[0].Type != clientv3.EventTypeDelete {
@@ -935,9 +911,9 @@ func TestWatchWithFilter(t *testing.T) {
 // TestWatchWithCreatedNotification checks that WithCreatedNotify returns a
 // Created watch response.
 func TestWatchWithCreatedNotification(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	cluster := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	cluster := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer cluster.Terminate(t)
 
 	client := cluster.RandClient()
@@ -957,9 +933,9 @@ func TestWatchWithCreatedNotification(t *testing.T) {
 // a watcher with created notify does not post duplicate
 // created events from disconnect.
 func TestWatchWithCreatedNotificationDropConn(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	cluster := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1, UseBridge: true})
+	cluster := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1, UseBridge: true})
 	defer cluster.Terminate(t)
 
 	client := cluster.RandClient()
@@ -986,9 +962,9 @@ func TestWatchWithCreatedNotificationDropConn(t *testing.T) {
 
 // TestWatchCancelOnServer ensures client watcher cancels propagate back to the server.
 func TestWatchCancelOnServer(t *testing.T) {
-	integration.BeforeTest(t)
+	integration2.BeforeTest(t)
 
-	cluster := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	cluster := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer cluster.Terminate(t)
 
 	client := cluster.RandClient()
@@ -1029,9 +1005,7 @@ func TestWatchCancelOnServer(t *testing.T) {
 	time.Sleep(time.Second)
 
 	minWatches, err := cluster.Members[0].Metric("etcd_debugging_mvcc_watcher_total")
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
 
 	maxWatchV, minWatchV := 0, 0
 	n, serr := fmt.Sscanf(maxWatches+" "+minWatches, "%d %d", &maxWatchV, &minWatchV)
@@ -1054,20 +1028,20 @@ func TestWatchCancelOnServer(t *testing.T) {
 //  4. watcher client finishes tearing down stream on "ctx"
 //  5. w2 comes back canceled
 func TestWatchOverlapContextCancel(t *testing.T) {
-	f := func(clus *integration.ClusterV3) {}
+	f := func(clus *integration2.Cluster) {}
 	testWatchOverlapContextCancel(t, f)
 }
 
 func TestWatchOverlapDropConnContextCancel(t *testing.T) {
-	f := func(clus *integration.ClusterV3) {
+	f := func(clus *integration2.Cluster) {
 		clus.Members[0].Bridge().DropConnections()
 	}
 	testWatchOverlapContextCancel(t, f)
 }
 
-func testWatchOverlapContextCancel(t *testing.T, f func(*integration.ClusterV3)) {
-	integration.BeforeTest(t)
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1, UseBridge: true})
+func testWatchOverlapContextCancel(t *testing.T, f func(*integration2.Cluster)) {
+	integration2.BeforeTest(t)
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1, UseBridge: true})
 	defer clus.Terminate(t)
 
 	n := 100
@@ -1082,10 +1056,11 @@ func testWatchOverlapContextCancel(t *testing.T, f func(*integration.ClusterV3))
 
 	// issue concurrent watches on "abc" with cancel
 	cli := clus.RandClient()
-	if _, err := cli.Put(context.TODO(), "abc", "def"); err != nil {
-		t.Fatal(err)
-	}
+	_, err := cli.Put(context.TODO(), "abc", "def")
+	require.NoError(t, err)
 	ch := make(chan struct{}, n)
+	tCtx, cancelFunc := context.WithCancel(context.Background())
+	defer cancelFunc()
 	for i := 0; i < n; i++ {
 		go func() {
 			defer func() { ch <- struct{}{} }()
@@ -1093,6 +1068,12 @@ func testWatchOverlapContextCancel(t *testing.T, f func(*integration.ClusterV3))
 			ctx, cancel := context.WithCancel(ctxs[idx])
 			ctxc[idx] <- struct{}{}
 			wch := cli.Watch(ctx, "abc", clientv3.WithRev(1))
+			select {
+			case <-tCtx.Done():
+				cancel()
+				return
+			default:
+			}
 			f(clus)
 			select {
 			case _, ok := <-wch:
@@ -1127,8 +1108,8 @@ func testWatchOverlapContextCancel(t *testing.T, f func(*integration.ClusterV3))
 // TestWatchCancelAndCloseClient ensures that canceling a watcher then immediately
 // closing the client does not return a client closing error.
 func TestWatchCancelAndCloseClient(t *testing.T) {
-	integration.BeforeTest(t)
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	integration2.BeforeTest(t)
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer clus.Terminate(t)
 	cli := clus.Client(0)
 	ctx, cancel := context.WithCancel(context.Background())
@@ -1146,9 +1127,7 @@ func TestWatchCancelAndCloseClient(t *testing.T) {
 		}
 	}()
 	cancel()
-	if err := cli.Close(); err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, cli.Close())
 	<-donec
 	clus.TakeClient(0)
 }
@@ -1157,8 +1136,8 @@ func TestWatchCancelAndCloseClient(t *testing.T) {
 // to put them in resuming mode, cancels them so some resumes by cancel fail,
 // then closes the watcher interface to ensure correct clean up.
 func TestWatchStressResumeClose(t *testing.T) {
-	integration.BeforeTest(t)
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1, UseBridge: true})
+	integration2.BeforeTest(t)
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1, UseBridge: true})
 	defer clus.Terminate(t)
 	cli := clus.Client(0)
 
@@ -1170,17 +1149,15 @@ func TestWatchStressResumeClose(t *testing.T) {
 	}
 	clus.Members[0].Bridge().DropConnections()
 	cancel()
-	if err := cli.Close(); err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, cli.Close())
 	clus.TakeClient(0)
 }
 
 // TestWatchCancelDisconnected ensures canceling a watcher works when
 // its grpc stream is disconnected / reconnecting.
 func TestWatchCancelDisconnected(t *testing.T) {
-	integration.BeforeTest(t)
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	integration2.BeforeTest(t)
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	defer clus.Terminate(t)
 	cli := clus.Client(0)
 	ctx, cancel := context.WithCancel(context.Background())
@@ -1217,8 +1194,8 @@ func testWatchClose(t *testing.T, wctx *watchctx) {
 }
 
 func TestWatch(t *testing.T) {
-	integration.BeforeTest(t)
-	clus := integration.NewClusterV3(t, &integration.ClusterConfig{Size: 1})
+	integration2.BeforeTest(t)
+	clus := integration2.NewCluster(t, &integration2.ClusterConfig{Size: 1})
 	t.Cleanup(func() { clus.Terminate(t) })
 	client := clus.Client(0)
 
@@ -1259,12 +1236,15 @@ func TestWatch(t *testing.T) {
 	t.Log("Validate")
 	for i, tc := range tcs {
 		t.Run(tc.name, func(t *testing.T) {
-			_, err := collectEvents(ctx, watches[i])
+			events, err := collectEvents(ctx, watches[i])
 			if tc.wantError == nil {
 				require.NoError(t, err)
 			} else {
 				require.ErrorContains(t, err, tc.wantError.Error())
 			}
+			if diff := cmp.Diff(tc.wantEvents, events); diff != "" {
+				t.Errorf("unexpected events (-want +got):\n%s", diff)
+			}
 		})
 	}
 }
