@@ -13,13 +13,12 @@
 // limitations under the License.
 
 //go:build darwin
-// +build darwin
 
 package main
 
 import (
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"os"
 	"os/exec"
@@ -39,13 +38,13 @@ func install(ver, dir string) (string, error) {
 	}
 	defer resp.Body.Close()
 
-	d, err := ioutil.ReadAll(resp.Body)
+	d, err := io.ReadAll(resp.Body)
 	if err != nil {
 		return "", err
 	}
 
 	zipPath := filepath.Join(dir, "etcd.zip")
-	if err = ioutil.WriteFile(zipPath, d, fileutil.PrivateFileMode); err != nil {
+	if err = os.WriteFile(zipPath, d, fileutil.PrivateFileMode); err != nil {
 		return "", err
 	}
 
